function createDomainInputField(options) {
	var domain = options.domain;
	var classname = domain.classname;
	
	options.target.html('');
	
	if (options.label !== undefined) {
		var labelDiv = $('<label></label>');
		labelDiv.attr('for', options.idPrefix+'_value');
		labelDiv.html(options.label);
		
		options.target.append(labelDiv);
	}
	
	var valueContainer = $('<div></div>');

	if (classname.match(/.*\.RealDomain$/)) {
		createRealDomainInputField({
			'idPrefix' : options.idPrefix,
			'target' : valueContainer,
			'domain' : domain,
			'fillWithDefaults' : options.fillWithDefaults
		});
	} else if (classname.match(/.*\.IntegerDomain$/)) {
		createIntegerDomainInputField({
			'idPrefix' : options.idPrefix,
			'target' : valueContainer,
			'domain' : domain,
			'fillWithDefaults' : options.fillWithDefaults
		});
	} else if (classname.match(/.*\.CategoricalDomain$/)) {
		createCategoricalDomainInputField({
			'idPrefix' : options.idPrefix,
			'target' : valueContainer,
			'domain' : domain,
			'fillWithDefaults' : options.fillWithDefaults
		});
	} else if (classname.match(/.*\.BooleanDomain$/)) {
		createBooleanDomainInputField({
			'idPrefix' : options.idPrefix,
			'target' : valueContainer,
			'domain' : domain,
			'fillWithDefaults' : options.fillWithDefaults
		});
	} else {
		createUnknownDomainInputField({
			'idPrefix' : options.idPrefix,
			'target' : valueContainer,
			'domain' : domain,
			'fillWithDefaults' : options.fillWithDefaults
		})
	}
	
	options.target.append(valueContainer);
}

function createRealDomainInputField(options) {
	var field = $('<input></input>');
	field.attr('id', options.idPrefix+'_value');
	field.attr('name', options.idPrefix+'_value');
	field.attr('type', 'text');
	field.attr('size', 35);
	
	if (options.fillWithDefaults) {
		var defaultValue = options.domain['default']
		field.val(defaultValue);
	}
	
	options.target.html(field);
	
	return field;
}

function createIntegerDomainInputField(options) {
	var field = $('<input></input>');
	field.attr('id', options.idPrefix+'_value');
	field.attr('name', options.idPrefix+'_value');
	field.attr('type', 'text');
	field.attr('size', 35);
	
	if (options.fillWithDefaults) {
		var defaultValue = options.domain['default']
		field.val(defaultValue);
	}
	
	options.target.html(field);
}

function createCategoricalDomainInputField(options) {
	var field = $('<select></select>');
	field.attr('id', options.idPrefix+'_value');
	field.attr('name', options.idPrefix+'_value');

	var defaultValue = options.domain['default']

	for (var i in options.domain.items) {
		var opt = $('<option></option>');
		opt.attr('value', options.domain.items[i]);
		opt.html(options.domain.items[i]);
		
		if (options.fillWithDefaults && options.domain.items[i] === defaultValue) {
			opt.attr('selected', true);
		}
		
		field.append(opt);
	}
	
	options.target.html(field);
}

function createBooleanDomainInputField(options) {
	var field = $('<select></select>');
	field.attr('id', options.idPrefix+'_value');
	field.attr('name', options.idPrefix+'_value');

	var defaultValue = options.domain['default']

	var trueOpt = $('<option></option>');
	trueOpt.attr('value', 'true');
	trueOpt.html('true');
		
	if (options.fillWithDefaults && defaultValue === 'true') {
		trueOpt.attr('selected', true);
	}
		
	field.append(trueOpt);
	
	var falseOpt = $('<option></option>');
	falseOpt.attr('value', 'false');
	falseOpt.html('false');
		
	if (options.fillWithDefaults && defaultValue === 'false') {
		falseOpt.attr('selected', true);
	}
		
	field.append(falseOpt);
	
	options.target.html(field);
}

function createUnknownDomainInputField(options) {
	var field = $('<input></input>');
	field.attr('id', options.idPrefix+'_value');
	field.attr('name', options.idPrefix+'_value');
	field.attr('type', 'text');
	field.attr('size', 75);
	
	if (options.fillWithDefaults) {
		var defaultValue = options.domain['default']
		field.val(defaultValue);
	}
	
	options.target.html(field);
}

HAL.domainInput = function(options) {
	if ( !(this instanceof HAL.domainInput) ) {
	      return new HAL.domainInput(options);
	}
	
	var _idPrefix = options.idPrefix == null ? 'noid' : options.idPrefix;
	var _description = options.description == null ? 'No description' : options.description;
	var _domain = options.domain == null ? {} : options.domain;
	var _fillWithDefaults = options.fillWithDefaults == null ? true : options.fillWithDefaults;
	
	var _container = $('<div></div>');
	_container.addClass('formElement');
	
	createDomainInputField({
		target : _container,
		label : _description,
		domain : _domain,
		fillWithDefaults : _fillWithDefaults,
		idPrefix : _idPrefix
	});
	
	return {
		getElement : function() {
			return _container;
		},
		getValue : function() {
			var inputElement = _container.find('#'+_idPrefix+'_value');
			return inputElement.val();
		},
		setValue : function(value) {
			var inputElement = _container.find('#'+_idPrefix+'_value');
			inputElement.val(value);
		},
		update : function(newDomain, successCallback, failureCallback) {
			_domain = newDomain;
			
			var oldValue = this.getValue();
			
			_container.empty();
			
			createDomainInputField({
				target : _container,
				label : _description,
				domain : _domain,
				fillWithDefaults : _fillWithDefaults,
				idPrefix : _idPrefix
			});
		}
	};
};