function createEnvironmentSelector(options) {
	$.post('/hal/getFormSnippet/utilities/getDefinedEnvironments',
			{
				'hal_httpResponseContentType' : 'json'
			},
			function(response) {
				if (response.successful) {
					var environmentNames = response.result.environments
					options.target.html('');

					var defaultOpt = $('<option></option>');
					defaultOpt.attr('value', '');
					defaultOpt.html('--- Choose an execution environment ---');
					options.target.append(defaultOpt);

					for (var i in environmentNames) {
						var opt = $('<option></option>');
						opt.attr('value', environmentNames[i]);
						opt.html(environmentNames[i]);

						options.target.append(opt);
					}
				}
				
				options.onFinished({
					'successful' : response.successful
				});
			},
			'json'
		);
}

HAL.environmentSelector = function(options) {
	if ( !(this instanceof HAL.environmentSelector) ) {
	      return new HAL.environmentSelector(options);
	}

	var initialValue = options.initialValue;
	
	
	var _environmentLabel = $('<label></label>');
	_environmentLabel.attr('for', options.idPrefix+'_environmentName');
	_environmentLabel.html('Environment');
	
	var _environmentField = $('<select></select>');
	_environmentField.attr('name', options.idPrefix+'_environmentName');
	_environmentField.attr('id', options.idPrefix+'_environmentName');
	_environmentField.append($('<option></option>').attr('value', '').html('Populating...'));
	
	var _environmentContainer = $('<div></div>');
	_environmentContainer.addClass('formElement');
	if (options.showLabel == null || options.showLabel) {
		_environmentContainer.append(_environmentLabel);
	}
	_environmentContainer.append(_environmentField);
	
	var _changeCallbacks = [];
	_environmentField.change(function() {
		for (var i in _changeCallbacks) {
			_changeCallbacks[i]();
		}
	});
	
	$.post('/hal/getFormSnippet/utilities/getDefinedEnvironments',
		{
			'hal_httpResponseContentType' : 'json'
		},
		function(response) {
			if (response.successful) {
				var environmentNames = response.result.environments;
				_environmentField.empty();

				var defaultOpt = $('<option></option>');
				defaultOpt.attr('value', '');
				defaultOpt.html('--- Choose an execution environment ---');
				_environmentField.append(defaultOpt);

				for (var i in environmentNames) {
					var opt = $('<option></option>');
					opt.attr('value', environmentNames[i]);
					opt.html(environmentNames[i]);

					_environmentField.append(opt);
				}
				
				if (initialValue) {
					_environmentField.val(initialValue);
				}
			}
		},
		'json'
	);
	
	return {
		getElement : function() {
			return _environmentContainer;
		},
		addChangeCallback : function(callback) {
			_changeCallbacks.push(callback);
		},
		getValue : function() {
			return _environmentField.val();
		},
		setValue: function(value) {
			_environmentField.children().each(function(index,item) {
				if ($(item).val() == value) { _environmentField.val(value); }
			});
		}
	};
};

HAL.defaultExecutionEnvironmentMenu = function(options) {
	if ( !(this instanceof HAL.defaultExecutionEnvironmentMenu) ) {
	      return new HAL.defaultExecutionEnvironmentMenu(options);
	}
	
	var _headerContent = options.headerContent == null ? 'Execution environment' : options.headerContent;
	var _idPrefix = options.idPrefix == null ? 'noid' : options.idPrefix;
	var _isMenuEnabled = options.enabled == null ? true : options.enabled;
	var _isMenuExpanded = options.expanded == null ? true : options.expanded;
	var _showPrevious = options.showPrevious == null ? true : options.showPrevious;
	var _showNext = options.showNext == null ? true : options.showNext;
	
	var _menu = new HAL.menu({
		header : _headerContent,
		enabled : _isMenuEnabled,
		expanded : _isMenuExpanded,
		showPrevious : _showPrevious,
		showNext : _showNext
	});
		
	var _environmentField = new HAL.environmentSelector({
		idPrefix : _idPrefix,
		initialValue : (!options.defaults) ? undefined : options.defaults.environment,
	});
	
	_menu.getContentElement().append(_environmentField.getElement());
	
	return {
		getMenuElement : function() {
			return _menu.getMenuElement();
		},
		getMenuObject : function() {
			return _menu;
		},
		getContentElement : function() {
			return _menu.getContentElement();
		},
		setDefaults : function(defaults) {
			_environmentField.setValue(defaults.environment);
		},
		enable : function() {
			_menu.enable();
		},
		disable : function() {
			_menu.disable();
		},
		expand : function() {
			_menu.expand();
		},
		collapse : function() {
			_menu.collapse();
		},
		setBusy : function(isBusy) {
			_menu.setBusy(isBusy);
		},
		addOnNext : function(f) {
			_menu.addOnNext(f);
		},
		addOnPrevious : function(f) {
			_menu.addOnPrevious(f);
		},
		clearMessages : function() {
			_menu.clearMessages();
		},
		addSuccessMessage : function(msg) {
			_menu.addSuccessMessage(msg);
		},
		addWarningMessage : function(msg) {
			_menu.addWarningMessage(msg);
		},
		addErrorMessage : function(msg) {
			_menu.addErrorMessage(msg);
		},
		addNavigationElement : function(element) {
			_menu.addNavigationElement(element);
		},
		getEnvironment : function() {
			return _environmentField.getValue();
		},
		addEnvironmentChangeCallback : function(callback) {
			_environmentField.addChangeCallback(callback);
		}
	};
};