function createInstanceFeatureForm(options) {
	var newInstanceFeatureForm = $('<form></form>');
	newInstanceFeatureForm.attr('id', 'newFeature');
	
	var featureNameLabel = $('<label></label>');
	featureNameLabel.attr('for', 'newFeature_featureName');
	featureNameLabel.html('Feature name');
	
	var featureNameField = $('<input></input>');
	featureNameField.attr('type', 'text');
	featureNameField.attr('size', 50);
	featureNameField.attr('id', 'newFeature_featureName');
	featureNameField.attr('name', 'newFeature_featureName');
	
	var featureNameContainer = $('<div></div>');
	featureNameContainer.addClass('formElement');
	featureNameContainer.append(featureNameLabel);
	featureNameContainer.append(featureNameField);
	
	newInstanceFeatureForm.append(featureNameContainer);
	
	var tagSetsHeaderText = $('<span></span>').html('Tag sets');
	var tagSetsContent = $('<div></div>');
	var tagSetsContainer = $('<div></div>');

	createCollapsibleMenu({
		'headerContent' : tagSetsHeaderText,
		'contentContainer' : tagSetsContent,
		'menuContainer' : tagSetsContainer,
		'initialState' : 'expanded'
	});
	
	createMultipleTagSetSelector({
		'target' : tagSetsContent,
		'idPrefix' : 'newFeature_',
		'description' : 'Please specify one or more sets of tags, such that one set must be present on an instance in order to use this feature. Each tag set should be a comma-separated list of tags, and you may use the drop-down selector to insert tags that are already defined in HAL.'
	});
	
	newInstanceFeatureForm.append(tagSetsContainer);
	
	var domainHeaderText = $('<span></span>').html('Domain');
	var domainContent = $('<div></div>');
	var domainContainer = $('<div></div>');
	
	createCollapsibleMenu({
		'headerContent' : domainHeaderText,
		'contentContainer' : domainContent,
		'menuContainer' : domainContainer,
		'initialState' : 'expanded'
	});
	
	createParameterDomainSpecification({
		'target' : domainContent,
		'parameter' : undefined,
		'idPrefix' : 'newFeature_domain_',
		'showDefaultValueField' : false,
		'showTransformationField' : true
	});
	
	newInstanceFeatureForm.append(domainContainer);
	
	var newInstanceFeatureStatusMessages = $('<div></div>');
	
	var submitButton = $('<input></input>');
	submitButton.attr('type', 'button');
	submitButton.attr('value', 'Submit');
	submitButton.click(function() {
		submitButton.attr('disabled', true);
		
		newInstanceFeatureStatusMessages.removeClass('error').removeClass('alert').addClass('success');
		var spinner = $('<div></div>').addClass('center').addClass('spinner');
		spinner.append($('<img>').attr('src', '/hal/commands/getFile/web/icons/ajax-loader.gif').addClass('center'));
		newInstanceFeatureStatusMessages.html(spinner);
		
		var formData = newInstanceFeatureForm.serialize();
		formData = formData+'&hal_httpResponseContentType=json';
		
		$.post('/hal/getFormSnippet/instanceFeature/createInstanceFeature',
			formData,
			function(response) {
				submitButton.attr('disabled', false);
				
				if (response.successful) {
					newInstanceFeatureStatusMessages.removeClass('error').removeClass('alert').addClass('success');
					newInstanceFeatureStatusMessages.html('Instance feature defined successfully.');
					newInstanceFeatureStatusMessages.show();
					
					setTimeout("window.location.href = '/hal/'", 1000);
				} else {
					newInstanceFeatureStatusMessages.removeClass('success').removeClass('alert').addClass('error');
					newInstanceFeatureStatusMessages.html(response.errorMessage);
					newInstanceFeatureStatusMessages.show();
				}
			},
			'json'
		);
	});
	
	var cancelButton = $('<input></input>');
	cancelButton.attr('type', 'button');
	cancelButton.attr('value', 'Cancel');
	cancelButton.click(cancelForm);
	
	var newInstanceFeatureButtons = $('<div></div>');
	newInstanceFeatureButtons.addClass('center');
	newInstanceFeatureButtons.append(submitButton);
	newInstanceFeatureButtons.append(cancelButton);
	
	options.target.append(newInstanceFeatureForm);
	options.target.append(newInstanceFeatureStatusMessages);
	options.target.append('<hr>');
	options.target.append(newInstanceFeatureButtons);
}

function createAddFeaturesToInstanceSetForm(options) {
	var addFeaturesForm = $('<form></form>');
	addFeaturesForm.attr('id', 'addFeaturesToInstanceSet');
	
	var instanceSetSelectionLabel = $('<label></label>');
	instanceSetSelectionLabel.attr('for', 'addFeature_instanceSetName');
	instanceSetSelectionLabel.html('Instance set');
	
	var instanceSetSelectionDescription = $('<div></div>');
	instanceSetSelectionDescription.addClass('formFieldDescription');
	instanceSetSelectionDescription.html('');
	
	var instanceSetSelectionField = $('<select></select>');
	instanceSetSelectionField.attr('id', 'addFeature_instanceSetName');
	instanceSetSelectionField.attr('name', 'addFeature_instanceSetName');
	
	for (var i in options.instanceSets) {
		var opt = $('<option></option>');
		opt.attr('value', options.instanceSets[i]);
		opt.html(options.instanceSets[i]);
		
		instanceSetSelectionField.append(opt);
	}
	
	var instanceSetSelectionContainer = $('<div></div>');
	instanceSetSelectionContainer.addClass('formElement');
	instanceSetSelectionContainer.append(instanceSetSelectionLabel);
	instanceSetSelectionContainer.append(instanceSetSelectionDescription);
	instanceSetSelectionContainer.append(instanceSetSelectionField);
	
	addFeaturesForm.append(instanceSetSelectionContainer);
	
	var featuresHeaderText = $('<span></span>').html('Features');
	var featuresContent = $('<div></div>');
	var featuresContainer = $('<div></div>');
	
	createCollapsibleMenu({
		'headerContent' : featuresHeaderText,
		'contentContainer' : featuresContent,
		'menuContainer' : featuresContainer,
		'initialState' : 'collapsed'
	});
	
	disableMenu(featuresContainer);
	
	var instancesHeaderText = $('<span></span>').html('Select specific instances');
	var instancesContent = $('<div></div>');
	var instancesContainer = $('<div></div>');
	
	createCollapsibleMenu({
		'headerContent' : instancesHeaderText,
		'contentContainer' : instancesContent,
		'menuContainer' : instancesContainer,
		'initialState' : 'collapsed'
	});
	
	disableMenu(instancesContainer);
	
	addFeaturesForm.append(instancesContainer);
	addFeaturesForm.append(featuresContainer);
	
	instanceSetSelectionField.change(function() {	
		instancesContent.html('');
		
		featuresContent.html('');
		featuresContent.append($('<img>').attr('src', '/hal/commands/getFile/web/icons/ajax-loader.gif'));
		
		// Need to get all of the instance names or hashes to display in a collapsed list with checkboxes for inclusion,
		// features will get added only to those instances.
		$.post('/hal/getFormSnippet/utilities/getInstances',
			{
				'hal_httpResponseContentType' : 'json',
				'instanceSet' : instanceSetSelectionField.val()
			},
			function(response) {
				if (response.successful) {
					for (var i in response.result.instances) {
						var elementLabel = $('<label></label>');
						elementLabel.attr('for', 'addFeature_instance_'+response.result.instances[i].hash);
						elementLabel.html(response.result.instances[i].name);
						
						var elementCheckbox = $('<input></input>');
						elementCheckbox.attr('type', 'checkbox');
						elementCheckbox.attr('checked', 'on');
						elementCheckbox.attr('id', 'addFeature_instance_'+response.result.instances[i].hash);
						elementCheckbox.attr('name', 'addFeature_instance_'+response.result.instances[i].hash);
						
						var element = $('<div></div>');
						element.addClass('formElementInline');
						element.append(elementLabel);
						element.append(elementCheckbox);
						
						instancesContent.append(element);
					}
					
					enableMenu(instancesContainer);
					
					enableMenu(featuresContainer);
					expandMenu(featuresContainer);
					
					$.post('/hal/getFormSnippet/utilities/getCompatibleFeatures',
							{
								'hal_httpResponseContentType' : 'json',
								'instanceSet' : instanceSetSelectionField.val()
							},
							function(response) {
								if (response.successful) {
									featuresContent.html('');
									
									createMultipleFeatureSelector({
										'target' : featuresContent,
										'idPrefix' : 'addFeature_',
										'features' : response.result.compatibleFeatures
									});
								}
							},
							'json'
						);
				}
			},
			'json'
		);
			

	});
	
	instanceSetSelectionField.change();
	
	var addFeaturesStatusMessages = $('<div></div>');
	
	var submitButton = $('<input></input>');
	submitButton.attr('type', 'button');
	submitButton.attr('value', 'Add features');
	submitButton.click(function() {
		submitButton.attr('disabled', true);

		addFeaturesStatusMessages.removeClass('error').removeClass('alert').addClass('success');
		var spinner = $('<div></div>').addClass('center').addClass('spinner');
		spinner.append($('<img>').attr('src', '/hal/commands/getFile/web/icons/ajax-loader.gif').addClass('center'));
		addFeaturesStatusMessages.html(spinner);
		
		var formData = addFeaturesForm.serialize();
		formData = formData+'&hal_httpContentResponse=json';
		
		$.post('/hal/getFormSnippet/instanceFeature/addFeaturesToInstanceSet',
			formData,
			function(response) {
				submitButton.attr('disabled', false);
				
				if (response.successful) {
					addFeaturesStatusMessages.removeClass('error').removeClass('alert').addClass('success');
					addFeaturesStatusMessages.html('All features were added successfully.');
					addFeaturesStatusMessages.show();
					
					setTimeout("window.location.href = '/hal/';", 1000);
				} else {
					addFeaturesStatusMessages.removeClass('success').removeClass('alert').addClass('error');
					addFeaturesStatusMessages.html(response.errorMessage);
					addFeaturesStatusMessages.show();
				}
			},
			'json'
		);
	});
	
	var cancelButton = $('<input></input>');
	cancelButton.attr('type', 'button');
	cancelButton.attr('value', 'Cancel');
	cancelButton.click(cancelForm);
	
	var addFeaturesButtons = $('<div></div>');
	addFeaturesButtons.addClass('center');
	addFeaturesButtons.append(submitButton);
	addFeaturesButtons.append(cancelButton);
	
	options.target.append(addFeaturesForm);
	options.target.append(addFeaturesStatusMessages);
	options.target.append('<hr>');
	options.target.append(addFeaturesButtons);
}

function createFeatureExtractorSelector(options) {
	options.target.addClass('formElement');

	var extractorLabel = $('<label></label>');
	extractorLabel.attr('for', options.idPrefix+'_extractorName');
	extractorLabel.html('Feature extractor');
	
	var extractorField = $('<select></select>');
	extractorField.attr('id', options.idPrefix+'_extractorName');
	extractorField.attr('name', options.idPrefix+'_extractorName');

	var defaultOption = $('<option></option>');
	defaultOption.attr('value', '');
	if (numberOfProperties(options.featureExtractors) !== 0) {
		defaultOption.html('--- Select a feature extractor ---');
	} else {
		defaultOption.html('--- No compatible feature extractors! ---');
	}
	extractorField.append(defaultOption);
	
	for (var i in options.featureExtractors) {
		var opt = $('<option></option>');
		opt.attr('value', options.featureExtractors[i].name+'_xXxversionxXx_'+options.featureExtractors[i].version);
		
		var id = options.featureExtractors[i].name;
		if (options.featureExtractors[i].version !== null && options.featureExtractors[i].version !== '') {
			id = id+' ('+options.featureExtractors[i].version+')';
		}
		opt.html(id);
		
		extractorField.append(opt);
	}
	
	var removeExtractorButton = $('<input></input>');
	removeExtractorButton.attr('type', 'button');
	removeExtractorButton.attr('value', 'Remove');
	removeExtractorButton.click(function() {
		$(this).parent().remove();
	});
	
	options.target.append(extractorLabel);
	options.target.append(extractorField);
	options.target.append(removeExtractorButton);
}

function createMultipleFeatureExtractorSelector(options) {
	options.target.html('');
	var compatibleFeatureList = [];
	
	var outerContainer = $('<div></div>');
	outerContainer.hide();
	
	var description = $('<div></div>');
	description.addClass('formFieldDescription');
	description.html(options.description);
	outerContainer.append(description);
	
	var numExtractors = 0;
	var numExtractorsField = $('<input></input>');
	numExtractorsField.attr('type', 'hidden');
	numExtractorsField.attr('name', options.idPrefix+'numberOfFeatureExtractors');
	numExtractorsField.attr('id', options.idPrefix+'numberOfFeatureExtractors');
	numExtractorsField.attr('value', numExtractors);
	outerContainer.append(numExtractorsField);
	
	var extractors = $('<div></div>');
	outerContainer.append(extractors);
	
	var addClick = function() {
		var newExtractorDiv = $('<div></div>');
		extractors.append(newExtractorDiv);
		
		createFeatureExtractorSelector({
			'target' : newExtractorDiv,
			'featureExtractors' : compatibleFeatureList,
			'idPrefix' : options.idPrefix+'featureExtractors_'+numExtractors+'_',
			'removable' : true
		});
		
		numExtractors = numExtractors+1;
		numExtractorsField.val(numExtractors);
	}
	
	var addButton = $('<input></input>');
	addButton.attr('type', 'button');
	addButton.attr('value', 'Add feature extractor');
	addButton.click(addClick);
	
	var next = function() {
		var selectedExtractors = {};
		var extractedFeatures = {};
		
		extractors.find('select').each(function() {
			selectedExtractors[$(this).val()] = true;
		});
		
		for (var i in compatibleFeatureList) {
			var idString = compatibleFeatureList[i].name+'_xXxversionxXx_'+compatibleFeatureList[i].version;
			
			if (selectedExtractors[idString] === true) {
				for (var j in compatibleFeatureList[i].features) {
					extractedFeatures[compatibleFeatureList[i].features[j]] = true;
				}
			}
		}
		
		options.onNext(extractedFeatures);
	}
	
	var nextButton = $('<input></input>');
	nextButton.attr('type', 'button');
	nextButton.attr('value', 'Next');
	nextButton.click(next);
	
	var buttons = $('<div></div>');
	buttons.append('<hr>');
	buttons.append(addButton);
	buttons.append(nextButton);
	
	outerContainer.append(buttons);
	
	$.post('/hal/getFormSnippet/utilities/getCompatibleFeatureExtractors',
		{
			'hal_httpResponseContentType' : 'json',
			'instanceSet' : options.instanceSet
		},
		function(response) {
			if (response.successful) {
				compatibleFeatureList = response.result.compatibleFeatureExtractors
				
				outerContainer.show();
				options.target.html(outerContainer);
				options.onFinished(compatibleFeatureList);
			}
		},
		'json'
	);
}

function createMultipleFeatureSelector(options) {
	var featureContainer = $('<div></div>');
	
	var numFeatures = 0;
	var numFeaturesField = $('<input></input>');
	numFeaturesField.attr('type', 'hidden');
	numFeaturesField.attr('name', options.idPrefix+'numberOfFeatures');
	numFeaturesField.attr('id', options.idPrefix+'numberOfFeatures');
	numFeaturesField.attr('value', numFeatures);
	
	var addFeatureButton = $('<input></input>');
	addFeatureButton.attr('id', options.idPrefix+'addFeatureButton');
	addFeatureButton.attr('type', 'button');
	addFeatureButton.attr('value', 'Add feature');
	addFeatureButton.click(function(){
		var newFeatureDiv = $('<div></div>');
		featureContainer.append(newFeatureDiv);
		
		createFeatureSelector({
			'target' : newFeatureDiv,
			'idPrefix' : options.idPrefix+'feature_'+numFeatures+'_',
			'features' : options.features,
			'removable' : true
		});
		
		numFeatures = numFeatures+1;
		numFeaturesField.val(numFeatures);
	});
	
	options.target.append(featureContainer);
	options.target.append(numFeaturesField);
	options.target.append(addFeatureButton);
}

function createFeatureSelector(options) {
	var featureLabel = $('<label></label>');
	featureLabel.attr('for', options.idPrefix+'featureName');
	featureLabel.html('Feature');
	
	var featureField = $('<select></select>');
	featureField.attr('id', options.idPrefix+'featureName');
	featureField.attr('name', options.idPrefix+'featureName');
	
	for (var i in options.features) {
		var opt = $('<option></option>');
		opt.attr('value', options.features[i]);
		opt.html(options.features[i]);
		
		featureField.append(opt);
	}
	
	var removeFeatureButton = $('<input></input>');
	removeFeatureButton.attr('type', 'button');
	removeFeatureButton.attr('value', 'Remove');
	removeFeatureButton.click(function() {
		$(this).parent().parent().remove();
	});
	
	var featureSelectionContainer = $('<div></div>');
	featureSelectionContainer.addClass('formElement');
	
	featureSelectionContainer.append(featureLabel);
	featureSelectionContainer.append(featureField);
	featureSelectionContainer.append(removeFeatureButton);
	
	var featureValueLabel = $('<label></label>');
	featureValueLabel.attr('for', options.idPrefix+'featureValue');
	featureValueLabel.html('Value');
	
	var featureValueField = $('<input></input>');
	featureValueField.attr('id', options.idPrefix+'featureValue');
	featureValueField.attr('name', options.idPrefix+'featureValue');
	featureValueField.attr('type', 'text');
	featureValueField.attr('size', 50);
	
	var featureValueContainer = $('<div></div>');
	featureValueContainer.addClass('formElement');
	featureValueContainer.append(featureValueLabel);
	featureValueContainer.append(featureValueField);
	
	options.target.append(featureSelectionContainer);
	options.target.append(featureValueContainer);
}

HAL.multipleFeatureExtractorSelector = function(options) {
	if ( !(this instanceof HAL.multipleFeatureExtractorSelector) ) {
	      return new HAL.multipleFeatureExtractorSelector(options);
	}
	
	var _idPrefix = options.idPrefix == null ? 'noid' : options.idPrefix;
	var _featureExtractors = options.featureExtractors == null ? {} : options.featureExtractors;
	var _minimum = options.minimum == null ? 0 : options.minimum;
	var _maximum = options.maximum;
	
	// context-sensitive description for the user.
	var _help = $('<div></div>');
	_help.addClass('formFieldDescription');
	if (_minimum == _maximum) {
		_help.html('Please select '+_minimum+' feature extractor');
		
		if (_minimum != 1) {
			_help.append('s');
		}
		_help.append('.');
	} else if (_maximum == null) {
		_help.html('Please select '+_minimum+' or more feature extractors.');
	} else {
		_help.html('Please select between '+_minimum+' and '+_maximum+' feature extractors.');
	}
	
	var _extractorElements = [];
	var _extractorContainer = $('<div></div>');
	
	var _numFeatureExtractors = 0;
	var _numFeatureExtractorsField = $('<input></input>');
	_numFeatureExtractorsField.attr('type', 'hidden');
	_numFeatureExtractorsField.attr('name', _idPrefix+'_numberOfFeatureExtractors');
	_numFeatureExtractorsField.attr('id', _idPrefix+'_numberOfFeatureExtractors');
	_numFeatureExtractorsField.attr('value', _numFeatureExtractors);
	
	var _addFeatureExtractor = function(addOptions) {
		var newFeatureExtractorDiv = $('<div></div>');
		_extractorContainer.append(newFeatureExtractorDiv);
		
		var _extractor = new HAL.featureExtractorSelectionElement({
			idPrefix : _idPrefix+'_'+_numFeatureExtractors,
			featureExtractors : _featureExtractors,
			isRemovable : addOptions.isRemovable
		});
		
		_extractorElements.push(_extractor);
		_extractorContainer.append(_extractor.getElement());
		
		_numFeatureExtractors = _numFeatureExtractors+1;
		_numFeatureExtractorsField.val(_numFeatureExtractors);
	};
	
	for (var i=0; i<_minimum; i++) {
		_addFeatureExtractor({
			isRemovable : false
		});
	}
	
	var _container = $('<div></div>');
	_container.append(_help);
	_container.append(_extractorContainer);
	_container.append(_numFeatureExtractorsField);
	
	return {
		getElement : function() {
			return _container;
		},
		getValues : function() {
			var values = {};
			
			for (var i in _extractorElements) {
				values[i] = _extractorElements[i].getElement().find('select').val();
			}
			
			return values;
		},
		addFeatureExtractor : function(addOptions) {
			_addFeatureExtractor(addOptions);
		},
		updateFeatureExtractorList : function(newExtractorList) {
			_featureExtractors = newExtractorList;
			
			for (var i in _extractorElements) {
				_extractorElements[i].updateFeatureExtractorList(_featureExtractors);
			}
		}
	};
};

HAL.featureExtractorSelectionElement = function(options) {
	if ( !(this instanceof HAL.featureExtractorSelectionElement) ) {
	      return new HAL.multipleFeatureExtractorSelectionElement(options);
	}
	
	var _idPrefix = options.idPrefix == null ? 'noid' : options.idPrefix;
	var _featureExtractors = options.featureExtractors == null ? {} : options.featureExtractors;
	var _isRemovable = options.isRemovable == null ? true : options.isRemovable;
	
	var _container = $('<div></div>');
		
	createFeatureExtractorSelector({
		'target' : _container,
		'featureExtractors' : _featureExtractors,
		'idPrefix' : _idPrefix,
		'removable' : _isRemovable
	});
		
	return {
		getElement : function() {
			return _container;
		},
		getValue : function() {
			return _container.find('select').val();
		},
		updateFeatureExtractorList : function(newExtractorList) {
			_featureExtractors = newExtractorList;
			
			var oldVal = _container.find('select').val();
			if (oldVal == null) {
				oldVal = '';
			}
			
			_container.empty();
			
			createFeatureExtractorSelector({
				'target' : _container,
				'featureExtractors' : _featureExtractors,
				'idPrefix' : _idPrefix+'_'+'extractorName',
				'removable' : _isRemovable
			});
				
			_container.find('select').val(oldVal);
		}
	};
};

HAL.multipleFeatureSubsetSelector = function(options) {
	if ( !(this instanceof HAL.multipleFeatureSubsetSelector) ) {
		return new HAL.multipleFeatureSubsetSelector;
	}
	
	var _idPrefix = options.idPrefix == null ? 'noid' : options.idPrefix;
	var _minimum = options.minimum == null ? 0 : options.minimum;
	var _maximum = options.maximum;
	var _defaultToSelected = options.defaultToSelected == null ? false : options.defaultToSelected;
	var _features = options.features == null ? {} : options.features;
	
	// context-sensitive description for the user.
	var _help = $('<div></div>');
	_help.addClass('formFieldDescription');
	if (_minimum == _maximum) {
		_help.html('Please select '+_minimum+' feature');
		
		if (_minimum != 1) {
			_help.append('s');
		}
		_help.append('.');
	} else if (_maximum == null) {
		_help.html('Please select '+_minimum+' or more features.');
	} else {
		_help.html('Please select between '+_minimum+' and '+_maximum+' features.');
	}
	
	var _selected = {};
	var _selectionContainer = $('<div></div>');
		
	var _addFeature = function(f, isSelected) {
		var label = $('<label></label>');
		label.attr('for', _idPrefix+'_'+f);
		label.html(f);
		
		var field = $('<input></input>');
		field.attr('id', _idPrefix+'_'+f);
		field.attr('name', _idPrefix+'_'+f);
		field.attr('type', 'checkbox');
		field.attr('checked', isSelected);
		field.change(function() {
			_selected[f] = !_selected[f];
		});
		
		var element = $('<div></div>');
		element.addClass('formElementInline');
		element.append(field);
		element.append(label);

		_selected[f] = isSelected;
		_selectionContainer.append(element);
	};
	
	for (var i in _features) {
		_addFeature(_features[i], _defaultToSelected);
	}
	
	var _container = $('<div></div>');
	_container.append(_help);
	_container.append(_selectionContainer);
	
	return {
		getElement : function() {
			return _container;
		},
		getMinimum : function() {
			return _minimum;
		},
		getMaximum : function() {
			return _maximum;
		},
		getSelectedFeatures : function() {
			return {};
		},
		updateFeatureList : function(newFeatureList) {
			_features = newFeatureList;
			
			_selectionContainer.empty();
			for (var i in _features) {
				if (_selected[_features[i]] == undefined || _selected[_features[i]] == null) {
					_selected[_features[i]] = _defaultToSelected;
				}
				
				_addFeature(_features[i], _selected[_features[i]]);
			}
		}
	};
};

HAL.multipleFeatureSelector = function(options) {
	if ( !(this instanceof HAL.multipleFeatureSelector) ) {
	      return new HAL.multipleFeatureSelector(options);
	}
	
	var _idPrefix = options.idPrefix == null ? 'noid' : options.idPrefix;
	var _minimum = options.minimum == null ? 0 : options.minimum;
	var _maximum = options.maximum;
	
	var _features = options.features;

	// context-sensitive description for the user.
	var _help = $('<div></div>');
	_help.addClass('formFieldDescription');
	if (_minimum == _maximum) {
		_help.html('Please select '+_minimum+' feature');
		
		if (_minimum != 1) {
			_help.append('s');
		}
		_help.append('.');
	} else if (_maximum == null) {
		_help.html('Please select '+_minimum+' or more features.');
	} else {
		_help.html('Please select between '+_minimum+' and '+_maximum+' features.');
	}
	
	var _featureElements = [];
	var _featureContainer = $('<div></div>');
	
	var _numFeatures = 0;
	var _numFeaturesField = $('<input></input>');
	_numFeaturesField.attr('type', 'hidden');
	_numFeaturesField.attr('name', _idPrefix+'_numberOfFeatures');
	_numFeaturesField.attr('id', _idPrefix+'_numberOfFeatures');
	_numFeaturesField.attr('value', _numFeatures);
	
	var _addFeature = function(addOptions) {
		var newFeature = new HAL.featureSelectionElement({
			idPrefix : _idPrefix+'_'+_numFeatures,
			features : _features,
			isRemovable : addOptions.isRemovable
		});
		
		_featureContainer.append(newFeature.getElement());
		_featureElements.push(newFeature);
		
		_numFeatures = _numFeatures+1;
		_numFeaturesField.val(_numFeatures);
	};
	
	for (var i=0; i<_minimum; i++) {
		_addFeature({
			isRemovable : false
		});
	}
		
	var _container = $('<div></div>');
	_container.append(_help);
	_container.append(_featureContainer);
	_container.append(_numFeaturesField);
	
	return {
		getElement : function() {
			return _container;
		},
		updateFeatureList : function(newFeatureList) {
			_features = newFeatureList;
			
			for (var i in _featureElements) {
				_featureElements[i].updateFeatureList(_features);
			}
		},
		addFeature : function(addOptions) {
			_addFeature(addOptions);
		}
	};
};

HAL.featureSelectionElement = function(options) {
	if ( !(this instanceof HAL.featureSelectionElement) ) {
	      return new HAL.featureSelectionElement(options);
	}
				
	var _idPrefix = options.idPrefix == null ? 'noid' : options.idPrefix;
	var _features = options.features == null ? {} : options.features;
	var _isRemovable = options.isRemovable == null ? true : options.isRemovable;
	
	var _container = $('<div></div>');
	
	createFeatureSelector({
		'target' : _container,
		'idPrefix' : _idPrefix,
		'features' : _features,
		'removable' : _isRemovable
	});
		
	return {
		getElement : function() {
			return _container;
		},
		updateFeatureList : function(newFeatureList) {
			_features = newFeatureList;
			
			var oldVal = _container.find('input').val();
			if (oldVal == null) {
				oldVal = '';
			}
			
			_container.empty();
			
			createFeatureSelector({
				'target' : _container,
				'idPrefix' : _idPrefix,
				'features' : _features,
				'removable' : _isRemovable
			});
			
			_container.find('input').val(oldVal);
		}
	};
};

HAL.defaultFeatureExtractionAndSelectionMenu = function(options) {
	if ( !(this instanceof HAL.defaultFeatureExtractionAndSelectionMenu) ) {
	      return new HAL.defaultFeatureExtractionAndSelectionMenu(options);
	}
	
	var _headerContent = options.headerContent == null ? 'Instance features' : options.headerContent;
	var _idPrefix = options.idPrefix == null ? 'noid' : options.idPrefix;
	var _isMenuEnabled = options.enabled == null ? true : options.enabled;
	var _isMenuExpanded = options.expanded == null ? true : options.expanded;
	var _minimumExtractors = options.minimumExtractors == null ? 0 : options.minimumExtractors;
	var _maximumExtractors = options.maximumExtractors;
	var _minimumFeatures = options.minimumFeatures == null ? 0 : options.minimumFeatures;
	var _maximumFeatures = options.maximumFeatures;
	var _defaultSelectAllFeatures = options.selectAllFeatures == null ? true : options.defaultSelectAllFeatures;
	var _compatibleFeatureExtractors = options.featureExtractors == null ? {} : options.featureExtractors;
	var _extractedFeatures = {};
	
	var _featureExtractionMenu = new HAL.menu({
		header : 'Feature extractor selection',
		enabled : true,
		expanded : true,
		showPrevious : true,
		showNext : true
	});
	
	var _featureSelectionMenu = new HAL.menu({
		header : 'Extracted features',
		enabled : false,
		expanded : false,
		showPrevious : true,
		showNext : true
	});
	
	var _menu = new HAL.menu({
		header : _headerContent,
		enabled : _isMenuEnabled,
		expanded : _isMenuExpanded,
		showPrevious : false,
		showNext : false
	});

	_menu.getContentElement().append(_featureExtractionMenu.getMenuElement());
	_menu.getContentElement().append(_featureSelectionMenu.getMenuElement());
	
	var _multipleFeatureExtractorSelector = new HAL.multipleFeatureExtractorSelector({
		idPrefix : _idPrefix+'_featureExtractors',
		minimum : _minimumExtractors,
		maximum : _maximumExtractors,
		featureExtractors : {}
	});
	_featureExtractionMenu.getContentElement().append(_multipleFeatureExtractorSelector.getElement());
	
	var _extractedFeatureSelector = new HAL.multipleFeatureSubsetSelector({
		idPrefix : _idPrefix+'_extractedFeatures',
		minimum : _minimumFeatures,
		maximum : _maximumFeatures,
		defaultToSelected : _defaultSelectAllFeatures,
		features : {}
	});
	_featureSelectionMenu.getContentElement().append(_extractedFeatureSelector.getElement());
	
	var addFeatureExtractorButton = $('<input></input>');
	addFeatureExtractorButton.attr('type', 'button');
	addFeatureExtractorButton.attr('value', 'Add feature extractor');
	addFeatureExtractorButton.click(function() {
		_multipleFeatureExtractorSelector.addFeatureExtractor({
			isRemovable : true
		});
	});
	_featureExtractionMenu.addNavigationElement(addFeatureExtractorButton);
	
	var updateFeatureSelection = function() {
		_featureSelectionMenu.setBusy(true);
		
		var selectedFeatureExtractors = _multipleFeatureExtractorSelector.getValues();

		var extractorIdsToIndex = {};
		for (var i in _compatibleFeatureExtractors) {
			var extractorId = _compatibleFeatureExtractors[i].name+'_xXxversionxXx_'+_compatibleFeatureExtractors[i].version;

			extractorIdsToIndex[extractorId] = i;
		}

		_extractedFeatures = [];
		var hasFeature = {};
		for (var i in selectedFeatureExtractors) {
			// check against the map
			// add extracted features to the set if we need to.
			if (selectedFeatureExtractors[i] !== '' &&
					extractorIdsToIndex[selectedFeatureExtractors[i]] !== null &&
					extractorIdsToIndex[selectedFeatureExtractors[i]] !== undefined &&
					extractorIdsToIndex[selectedFeatureExtractors[i]] !== '') {
		
				var index = extractorIdsToIndex[selectedFeatureExtractors[i]];
		
				var features = _compatibleFeatureExtractors[index].features;
				for (var j in features) {
					if (!hasFeature[features[j]]) {
						hasFeature[features[j]] = 1;
						_extractedFeatures.push(features[j]);
					}
				}
			}
		}

		_extractedFeatureSelector.updateFeatureList(_extractedFeatures);
		_featureSelectionMenu.setHeaderContent('Extracted features ('+numberOfProperties(_extractedFeatures)+')');
		
		_featureSelectionMenu.setBusy(false);
	};
	
	_featureExtractionMenu.addOnNext(function() {
		var vals = _multipleFeatureExtractorSelector.getValues();
		var valid = true;
		for (var i in vals) {
			if (vals[i] == '') {
				valid = false;
			}
		}
		
		if (valid) {
			_featureExtractionMenu.disable();
			
			_featureSelectionMenu.enable();
			_featureSelectionMenu.expand();
			
			updateFeatureSelection();
		}
	});
	
	_featureSelectionMenu.addOnPrevious(function() {
		_featureSelectionMenu.disable();
		
		_featureExtractionMenu.enable();
		_featureExtractionMenu.expand();
	});
	
	return {
		getMenuElement : function() {
			return _menu.getMenuElement();
		},
		getMenuObject : function() {
			return _menu;
		},
		getContentElement : function() {
			return _menu.getContentElement();
		},
		update : function(filterOptions, successCallback, failureCallback) {
			this.setBusy(true);
			
			var myMenu = this;
			$.post('/hal/getFormSnippet/utilities/getCompatibleFeatureExtractors',
				{
					'hal_httpResponseContentType' : 'json',
					'instanceSet' : filterOptions.compatibleWithInstanceDistribution
				},
				function(response) {
					if (response.successful) {
						_compatibleFeatureExtractors = response.result.compatibleFeatureExtractors;
						_multipleFeatureExtractorSelector.updateFeatureExtractorList(_compatibleFeatureExtractors);
						
						successCallback();
					} else {
						failureCallback();
					}
				
					myMenu.setBusy(false);
				},
				'json'
			);
		},
		enable : function() {
			_menu.enable();
		},
		disable : function() {
			_menu.disable();
		},
		expand : function() {
			_menu.expand();
		},
		collapse : function() {
			_menu.collapse();
		},
		setBusy : function(isBusy) {
			_menu.setBusy(isBusy);
		},
		addOnPrevious : function(f) {
			_featureExtractionMenu.addOnPrevious(f);
		},
		addOnNext : function(f) {
			_featureSelectionMenu.addOnNext(f);
		}
	};
};