$(document).ready(function() {
	// Builder class to build landing page menus
	var landingPageMenuBuilder = function() {
		var options = {};
		options.menu = {};
		options.menu.items = [];
		options.menu.content = {};
		
		var clear = function() {
			options = {};
			options.menu = {};
			options.menu.items = [];
			options.menu.content = {};
		}
		
		return {
			addMenuHeader: function(menuOptions) {
				// we expand upon HAL.menu's limited header options by
				// allowing the user to add two new fields:
				// @param id the id associated with this menu
				// @param classes[] any additional classes that should be added to the menu
				options.halMenuOptions = menuOptions;
				return this;
			},
			
			addContentOptions: function(contentOptions) {
				// can specify id and classes
				// @param id the id for the content div
				// @param classes[] the classes to be added to this div
				options.menu.content = contentOptions;
				return this;
			},
			
			addMenuItem: function(info) {
				// @expected info.href  where to link to
				// @expected info.label	text for anchor tags
				//
				// order is important when calling this method,
				// as it will dictate the order the li's will appear
				// when we add in the submenu items
				options.menu.items.push($('<li></li>').append($('<a></a>').attr('href',info.href).html(info.label)));
				return this;
			},
			
			buildMenu: function() {
				var menu = new HAL.menu(options.halMenuOptions);
				
				// handle content div options
				if (!$.isEmptyObject(options.menu.content)) {
					if (options.menu.content.id != undefined) {
						menu.getContentElement().parent().attr('id',options.menu.content.id);
					}
					
					if ((options.menu.content.classes != undefined)&&(options.menu.content.classes.length > 0)) {
						for(i in options.menu.content.classes) {
							menu.getContentElement().parent().addClass(options.menu.content.classes[i]);
						}
					}
				}
				
				// handle header menu element id/classes
				if (options.halMenuOptions.id != undefined) {
					menu.getMenuElement().attr('id',options.halMenuOptions.id);
				}
				
				if (options.halMenuOptions.classes != undefined) {
					for (i in options.halMenuOptions.classes) {
						menu.getMenuElement().addClass(options.halMenuOptions.classes[i]);
					}
				}
				
				// handle content items
				if (options.menu.items.length > 0) {
					var unorderedList = $('<ul></ul>');
					for(i in options.menu.items) {
						unorderedList.append(options.menu.items[i]);
					}				
					
					// clear out this menu after we've built a menu
					// we assume that any menus that want to be constructed
					// after buildMenu() is really a new menu
					clear();
					
					menu.getContentElement().append(unorderedList);
				}
				
				return menu.getMenuElement();
			}
		};
	}();
	
	//------------------------------------------------------------------
	//  Build first side menu on landing page
	//------------------------------------------------------------------
	
	var defineHALObjectTopLevelMenu = 
		landingPageMenuBuilder
		.addMenuHeader({ header: 'Define a HAL Object', expanded: true, enabled: true, id: 'halObjectDefinitionMenu' })
		.addMenuItem({ href: '/hal/createAlgorithm', label: 'Define a new algorithm' })
		.addMenuItem({ href: '/hal/createInstanceSet', label: 'Define a new instance set' })
		.addMenuItem({ href: '/hal/createInstanceFeature', label: 'Define a new instance feature' })
		.addMenuItem({ href: '/hal/addFeaturesToInstanceSet', label: 'Add features to an instance set' })
		.addMenuItem({ href: '/hal/createEnvironment', label: 'Define a new execution environment' })
		.buildMenu();
		//.attr('id','halObjectDefinitionMenu');
		
	var launchExperimentsTopLevelMenu = new HAL.menu({
		header: 'Launch an experiment',
		expanded: true,
		enabled: true
	});
	
	launchExperimentsTopLevelMenu.getMenuElement().attr('id','halRunExperimentMenu');
	
	var metaAlgorithmicAnalysisMenu = 
		landingPageMenuBuilder.addMenuHeader({ header: 'Meta-algorithmic analysis', enabled: true, expanded: true, id: 'halMetaAlgorithmicAnalysisMenu'  })
		.addMenuItem({ href: '/hal/createMultipleAlgorithmAnalysis', label: 'Analyse the performance of one or more algorithms' })
		.buildMenu();
		//.attr('id','halMetaAlgorithmicAnalysisMenu');
	
	
	var metaAlgorithmicDesignMenu = 
		landingPageMenuBuilder.addMenuHeader({ header: 'Meta-algorithmic design', enabled: true, expanded: true, id: 'halMetaAlgorithmicDesignMenu' })
		.addMenuItem({ href: '/hal/createSingleAlgorithmConfiguration', label: 'Optimize the performance of a single parameterized algorithm' })
		//.addMenuItem({ href: '/hal/createPortfolioBasedSelector', label: 'Create a portfolio-based algorithm selector' })
		.buildMenu();
		//.attr('id','halMetaAlgorithmicDesignMenu');
		
	launchExperimentsTopLevelMenu.getContentElement().append(metaAlgorithmicAnalysisMenu);
	launchExperimentsTopLevelMenu.getContentElement().append(metaAlgorithmicDesignMenu);
	
	var HALObjectManagementTopLevelMenu = 
		landingPageMenuBuilder.addMenuHeader({ header: 'Manage HAL objects', expanded: false, enabled: true, id: 'halManageObjectsMenu' })
		.addMenuItem({ href: '/hal/exportObjects', label: 'Export HAL objects' })
		.addMenuItem({ href: '/hal/importObjects', label: 'Import HAL objects' })
		.addMenuItem({ href: '/hal/uploadFile', label: 'Upload data for use in HAL' })
		.buildMenu();
		//.attr('id', 'halManageObjectsMenu');
	
	var HALDatabaseTopLevelMenu = 
		landingPageMenuBuilder.addMenuHeader({ header: 'View Database Items', expanded: false, enabled: true, id: 'halDatabaseMenu'})
		.addMenuItem({ href: '/jsp/Logs.jsp', label: 'View Log Records' })
		.buildMenu();
		
	//------------------------------------------------------------------
	// Stitching it all together
	//------------------------------------------------------------------
	
	$('#landingPageMenu').append(defineHALObjectTopLevelMenu)
	$('#landingPageMenu').append(launchExperimentsTopLevelMenu.getMenuElement());
	$('#landingPageMenu').append(HALObjectManagementTopLevelMenu);
	$('#landingPageMenu').append(HALDatabaseTopLevelMenu);
	
	//------------------------------------------------------------------
	//  Build second side menu on landing page
	//------------------------------------------------------------------
	
	var halRunStatusRunning = 
		landingPageMenuBuilder
		.addMenuHeader({ 
			header: 'Running (<span id=\"halNumRunsRunning\">loading...</span>)', 
			enabled: true, 
			expanded: true,
			id: 'halRunStatusRunning',
			classes: ['clear']
		})
		.addContentOptions({ id: 'halRunStatusContent_running', classes: ['runStatus'] })
		.buildMenu();
		//.attr('id','halRunStatusRunning')
		//.addClass('clear');
	
	var halRunStatusReadyToLaunch = 
		landingPageMenuBuilder
		.addMenuHeader({ 
			header: 'Ready to launch (<span id=\"halNumRunsReadyToLaunch\">loading...</span>)', 
			enabled: true, 
			expanded: false,
			id: 'halRunStatusReadyToLaunch',
			classes: ['clear']
		})
		.addContentOptions({ id: 'halRunStatusContent_readyToLaunch', classes: ['runStatus'] })
		.buildMenu();
		//.attr('id','halRunStatusReadyToLaunch')
		//.addClass('clear');
	
	var halRunStatusQueued = 
		landingPageMenuBuilder
		.addMenuHeader({ 
			header: 'Queued (<span id=\"halNumRunsQueued\">loading...</span>)', 
			enabled: true, 
			expanded: false,
			id: 'halRunStatusQueued',
			classes: ['clear']
		})
		.addContentOptions({ id: 'halRunStatusContent_queued', classes: ['runStatus'] })
		.buildMenu();
		//.attr('id','halRunStatusQueued')
		//.addClass('clear');
	
	var halRunStatusFinished = 
		landingPageMenuBuilder
		.addMenuHeader({ 
			header: 'Finished (<span id=\"halNumRunsFinished\">loading...</span>)', 
			enabled: true, 
			expanded: false,
			id: 'halRunStatusFinished',
			classes: ['clear']
		})
		.addContentOptions({ id: 'halRunStatusContent_finished', classes: ['runStatus'] })
		.buildMenu();
		//.attr('id','halRunStatusFinished')
		//.addClass('clear');
	
	var halRunStatusTerminatedByUser =
		landingPageMenuBuilder
		.addMenuHeader({ 
			header: 'Terminated by user (<span id=\"halNumRunsTerminatedByUser\">loading...</span>)', 
			enabled: true, 
			expanded: false,
			id: 'halRunStatusTerminatedByUser',
			classes: ['clear']
		})
		.addContentOptions({ id: 'halRunStatusContent_terminatedByUser', classes: ['runStatus'] })
		.buildMenu();
		//.attr('id','halRunStatusTerminatedByUser')
		//.addClass('clear');
	
	var halRunStatusTerminatedAbnormally = 
		landingPageMenuBuilder
		.addMenuHeader({ 
			header: 'Terminated abnormally (<span id=\"halNumRunsTerminatedAbnormally\">loading...</span>)', 
			enabled: true, 
			expanded: false,
			id: 'halRunStatusTerminatedAbnormally',
			classes: ['clear']
		})
		.addContentOptions({ id: 'halRunStatusContent_terminatedAbnormally', classes: ['runStatus'] })
		.buildMenu();
		//.attr('id', 'halRunStatusTerminatedAbnormally')
		//.addClass('clear');
	
	//------------------------------------------------------------------
	// Stitching it all together
	//------------------------------------------------------------------
	
	$('#runStatusMonitor')
		.append(halRunStatusRunning)
		.append(halRunStatusReadyToLaunch)
		.append(halRunStatusQueued)
		.append(halRunStatusFinished)
		.append(halRunStatusTerminatedByUser)
		.append(halRunStatusTerminatedAbnormally);
	
}); // end document.ready