HAL.menu = function(options) {
	if ( !(this instanceof HAL.menu) ) {
	      return new HAL.menu(options);
	}
	
	var _isMenuEnabled = options.enabled == null ? false : options.enabled;
	var _isMenuExpanded = options.expanded == null ? false : options.expanded;
	
	if (!_isMenuEnabled) {
		_isMenuExpanded = false;
	}
	
	var _showPrevious = options.showPrevious == null ? false : options.showPrevious;
	var _showNext = options.showNext == null ? false : options.showNext;
	var _otherNavigationElements = [];
	var _onPreviousCallbacks = [];
	var _onNextCallbacks = [];
	
	var _headerText = options.header;
		
	var _headerContent = $('<span></span>').html(_headerText);
	var _headerElement = $('<div></div>');	
	var _headerLink = $('<a></a>');
	
	_headerLink.append(_headerContent);
	_headerElement.append(_headerLink);
	
	var _contentMessagesAndNavigationElement = $('<div></div>');
	var _spinnerElement = $('<div></div>');
	var _contentElement = $('<div></div>');
	var _messagesElement = $('<div></div>');
	var _navigationElement = $('<div></div>');
	
	_spinnerElement.addClass('spinner');
	_spinnerElement.append($('<img>').attr('src', '/hal/commands/getFile/web/icons/ajax-loader.gif'));
	_spinnerElement.hide();
	
	var _previousButton = $('<input></input>');
	_previousButton.attr('type', 'button');
	_previousButton.attr('value', 'Previous');
	_previousButton.click(function() {
		for (var i in _onPreviousCallbacks) {
			_onPreviousCallbacks[i]();
		}
	});
	
	var _nextButton = $('<input></input>');
	_nextButton.attr('type', 'button');
	_nextButton.attr('value', 'Next');
	_nextButton.click(function() {
		for (var i in _onNextCallbacks) {
			_onNextCallbacks[i]();
		}
	});
	
	if (_showPrevious) {
		_navigationElement.append(_previousButton);
	}
	
	if (_showNext) {
		_navigationElement.append(_nextButton);
	}
	
	_contentMessagesAndNavigationElement.addClass('collapsibleContent');
	_contentMessagesAndNavigationElement.append(_spinnerElement);
	_contentMessagesAndNavigationElement.append(_contentElement);
	_contentMessagesAndNavigationElement.append(_messagesElement);
	_contentMessagesAndNavigationElement.append(_navigationElement);
	
	var _menuElement = $('<div></div>');
	
	_menuElement.addClass('collapsible');
	_menuElement.append(_headerElement);
	_menuElement.append(_contentMessagesAndNavigationElement);
	
	function toggleCollapse() {
		if (!_menuElement.hasClass('menuDisabled')) {
			_headerElement.toggleClass('collapsed').toggleClass('expanded');
			_contentMessagesAndNavigationElement.slideToggle('medium');
		}
	}
	
	if (!_isMenuEnabled) {
		_menuElement.addClass('menuDisabled');
	}
	
	if (_isMenuExpanded) {
		_headerElement.addClass('expanded');
	} else {
		_contentMessagesAndNavigationElement.hide();
		_headerElement.addClass('collapsed');
	}
	
	_headerLink.click(toggleCollapse);
	
	return {
		enable : function() {
			_isMenuEnabled = true;
			
			_menuElement.removeClass('menuDisabled');
		},
		disable : function() {
			_isMenuEnabled = false;
			
			this.collapse();
			_menuElement.addClass('menuDisabled');
		},
		expand : function() {
			if (!_menuElement.hasClass('menuDisabled')) {
				_isMenuExpanded = true;
				
				_headerElement.removeClass('collapsed').addClass('expanded');
				_contentMessagesAndNavigationElement.slideDown('medium', function() { _contentMessagesAndNavigationElement.show(); });
			}
		},
		collapse : function() {
			if (!_menuElement.hasClass('menuDisabled')) {
				_isMenuExpanded = false;
				
				_headerElement.removeClass('expanded').addClass('collapsed');
				_contentMessagesAndNavigationElement.slideUp('medium', function() { _contentMessagesAndNavigationElement.hide(); });
			}
		},
		addNavigationElement : function(element) {
			_otherNavigationElements.push(element);
			
			if (_showNext) {
				element.insertBefore(_nextButton);
			} else if (_showPrevious) {
				element.insertAfter(_previousButton);
			} else {
				_navigationElement.append(element);
			}
		},
		addOnPrevious : function(callback) {
			_onPreviousCallbacks.push(callback);
		},
		addOnNext : function(callback) {
			_onNextCallbacks.push(callback);
		},
		next : function() {
			_nextButton.click();
		},
		previous : function() {
			_previousButton.click();
		},
		setBusy : function(busy) {
			if (busy) {
				_contentElement.hide();
				_messagesElement.hide();
				_navigationElement.hide();
				_spinnerElement.show();
			} else {
				_spinnerElement.hide();
				_contentElement.show();
				_messagesElement.show();
				_navigationElement.show();
			}
		},
		setHeaderContent : function(text) {
			_headerContent.html(text);
		},
		addWarningMessage : function(message) {
			var warning = $('<div></div>');
			warning.addClass('warning');
			warning.append(message);
			
			_messagesElement.append(warning);
		},
		addErrorMessage : function(message) {
			var error = $('<div></div>');
			error.addClass('error');
			error.append(message);
			
			_messagesElement.append(error);
		},
		addSuccessMessage : function(message) {
			var success = $('<div></div>');
			success.addClass('success');
			success.append(message);
			
			_messagesElement.append(success);
		},
		clearMessages : function(message) {
			_messagesElement.empty();
		},
		getMenuElement : function() {
			return _menuElement;
		},
		getContentElement : function() {
			return _contentElement;
		},
		disablePreviousButton : function() {
			_previousButton.attr('disabled',true);
			return this;
		},
		enablePreviousButton : function() {
			_previousButton.attr('disabled',false);
			return this;
		},
		disableNextButton : function() {
			_nextButton.attr('disabled', true);
			return this;
		},
		enableNextButton : function() {
			_nextButton.attr('disabled',false);
			return this;
		},
		hidePreviousButton : function() {
			_previousButton.hide();
			return this;
		}
	};
}