$(document).ready(function() {
	function getURLParameter(name) {
		return decodeURIComponent(
			(location.search.match(RegExp("[?|&]"+name+'=(.+?)(&|$)'))||[,null])[1]
		);  
	}
	
	var queue = function() {
		var myQueue = [];
		var isBusy = false;
		
		return {
			enqueue: function(item) {
				// must wrap an Ajax call within a function()
				//
				// we do it this way so that we can have
				// direct access to the success function
				// which we have to modify to call
				// queue.run()
				//
				// this also has the added benefit of making the
				// enqueue chains look a bit nicer
				//
				// this also means that you don't have to remember
				// to put queue.run() at the end of the success
				// callback function, which I tend to do
				myQueue.push(function() {
					//alert(item.name);
					$.ajax({
						url: item.url,
						data: item.data,
						dataType: "text",
						success: function(data) {
							//alert("success");
							item.success(data);
							isBusy = false;
							queue.run();
						},
						
						complete: function(xhr, status) {
							//alert("complete: "+status);
							
						},
						error: function(jqXHR, textStatus, errorThrow) {
							//alert("jqXHR: "+jqXHR+" textStatus: "+textStatus+" errorThrown: "+errorThrown);
						}
						
					});
				});
				
				// try running the queue
				// if we aren't in the middle of an ajax call
				// then we should be able to run an ajax job
				//
				// this is for when the queue is empty, and we add
				// more jobs later
				
				queue.run();
				
				return this;
			},
			
			// already a wrapped function
//			enqueueWrapped: function(item) {
//				myQueue.push(function() {
//					item();
//					queue.run();
//				});
//				return this;
//				
//			},
			
			run: function() {
				
				if (myQueue.length > 0 && !isBusy) {
					isBusy = true;
					myQueue.shift()();
				}
			}
		}
	}();
	
//	function ajaxSubrunTable(fURL) {
//		var height = $('#subrunsData').height();
//		var clone = spinner.clone();
//		$('#subrunsData').html(clone.height(height));
//		
//		ajaxInProgress = true;
//		
//		$.ajax({
//			url: fURL,
//			dataType: 'html',
// 			success: function(data) {
// 				$('#subruns > div').html(data);
// 				ajaxInProgress = false;
// 				
// 				$('a.subrun').click(function() {
// 					if($(this).parent().hasClass("disabled")) return false;
// 					ajaxSubrunTable($(this).attr('href'));
// 					return false;
// 				});
// 				
// 				$('a.limit').click(function() {
// 					if($(this).parent().hasClass("disabled")) return false;
// 					ajaxSubrunTable($(this).attr('href'));
// 					return false;
// 				});
// 				
// 				$('a.autorefresh').click(function() {
// 					if ($(this).hasClass("active")) { $(this).removeClass("active"); shouldAutorefresh = false; }
// 					else { $(this).addClass("active"); shouldAutorefresh = true; }
// 					doRefreshing();
// 				});
// 			}
// 		});
// 	}
// 	
// 	function doRefreshing() {
// 		if(shouldAutorefresh) {
// 			refreshing();
// 			//changeAndSetTimeoutLength(1000);
// 		} else {
// 			cleaInterval(timer);
// 			var href=$('#firstPageLink').attr('href')+"&autorefresh=false";
// 			ajaxSubrunTable(href);
// 		}
// 	}
// 	
// 	function refreshing() {
// 		var href=$('#firstPageLink').attr('href')+"&autorefresh="+shouldAutorefresh;
// 		
// 		if (!ajaxInProgress) {
// 			if (timeoutLength <= 1000) { changeAndSetTimeoutLength(timeoutLength-100); }
// 			ajaxSubrunTable(href);
// 		} else if (ajaxInProgress && (numTimeouts<3)) {
// 			numTimeouts++;
// 		} else if (numTimeouts>=3) {
// 			jqHXR.abort();
// 			numTimeouts = 0;
// 			
// 			changeAndSetTimeoutLength(2*timeoutLength);
// 			
// 			ajaxSubrunTable(href);
// 		}
// 	}
// 
// 	
// 	function changeAndSetTimeoutLength(newLength) {
// 		if (newLength < 500) { timeoutLength = 500; }
// 		else { timeoutLength = newLength; }
// 		
// 		clearInterval(timer);
// 		timer = setInterval(refreshing,timeoutLength);
// 	}
	
	var spinner = 
		$('<div></div>')
		.addClass('spinner')
		.append($('<img></img>').attr('src','/hal/commands/getFile/web/icons/ajax-loader.gif'));

	var mainDivs = [];
	mainDivs.push($('#runSettings > div'))
	mainDivs.push($('#runStatus > div'))
	mainDivs.push($('#statistics > div'))
	mainDivs.push($('#bcistatistics > div'))
	mainDivs.push($('#scdPlot > div'));
	mainDivs.push($('#subruns > div'));

	for(var i in mainDivs) {
		mainDivs[i].html(spinner.clone());
	}
	
	var pageRunId = getURLParameter('runId');
	var timer;
	var shouldAutorefresh = false;
	var ajaxInProgress = false;
	
	queue
	.enqueue({
		name: 'metaAlgorithmRunSettings',
		url: '/hal/getFormSnippet/runStatusMonitoring/metaAlgorithmRunSettings',
		data: { runId: pageRunId },
		success: function(data) {
			$('#runSettings > div').html(data);
		}
	})
	.enqueue({
		name: 'metaAlgorithmRunStatus',
		url: '/hal/getFormSnippet/runStatusMonitoring/metaAlgorithmRunStatus',
		data: { runId: pageRunId },
		success: function(data) {
			$('#runStatus > div').html(data);
		}
	})
	.enqueue({
		name:  'metaAlgorithmRunStatistics',
		url: '/hal/getFormSnippet/runStatusMonitoring/metaAlgorithmRunStatistics',
		data: { runId: pageRunId },
		success: function(data) {
			$('#statistics > div').html(data);
		}
	})
	.enqueue({
		name: 'createPlot',
		url: '/hal/getFormSnippet/runStatusMonitoring/createPlot',
		data: { runId : pageRunId, plotType : 'SCD' },
		success: function(data) {
			$('#scdPlot > div').html(data);
		}
	});
	
	//.enqueue({
	//	name: 'bciMetaAlgorithmStatistics',
	//	url: '/hal/getFormSnippet/runStatusMonitoring/bciMetaAlgorithmStatistics',
	//	data: { runId: pageRunId, confidence : '' },
	//	success: function(data) {
	//		$('#bcistatistics > h4 > span').html(data);
	//	}
	
	//}).enqueue({
	//	name: 'bciMetaAlgorithmStatistics',
	//	url: '/hal/getFormSnippet/runStatusMonitoring/bciMetaAlgorithmStatistics',
	//	data: { runId: pageRunId },
	//	success: function(data) {
	//		$('#bcistatistics > div').html(data);
	//	}
	//})

	// cheat a bit here.  after much investigation, it would seem that
	// firebug can be a cause of an ajax error where the browser will
	// NOT registered an onreadystate == 4 change.  this issue was found
	// at https://bugzilla.mozilla.org/show_bug.cgi?id=343769.  inserting 
	// a small break -- like the one below -- allows Firebug enough time
	// to properly load itself and issue the onreadystate.
	//
	// after more investigation, it would seem that the problem is a
	// symptom of the option "show XMLHttpRequest" option under the 
	// console tab.  Unchecking this option also solves the problem.
	
	//setTimeout(function() { queue.run(); }, 1*1000);
	
	queue.run();
	
	var subruns = addSubrunsTableFunctionality(pageRunId);
	
	var runStatusAutoRefreshFunction = function() {
		queue
		.enqueue({
			name: 'metaAlgorithmRunStatus',
			url: '/hal/getFormSnippet/runStatusMonitoring/metaAlgorithmRunStatus',
			data: { runId: pageRunId },
			success: function(data) {
				$('#runStatus > div').html(data);
			}
		})
		.enqueue({
			name:  'metaAlgorithmRunStatistics',
			url: '/hal/getFormSnippet/runStatusMonitoring/metaAlgorithmRunStatistics',
			data: { runId: pageRunId },
			success: function(data) {
				$('#statistics > div').html(data);
			}
		})
		.enqueue({
			name: 'createPlot',
			url: '/hal/getFormSnippet/runStatusMonitoring/createPlot',
			data: { runId : pageRunId, plotType : 'SCD' },
			success: function(data) {
				$('#scdPlot > div').html(data);
			}
		});
		
	}
	
	var runStatusRefresh = setInterval(runStatusAutoRefreshFunction,5000);

});