HAL.singleAlgorithmConfigurationForm = function(options) {
	if ( !(this instanceof HAL.singleAlgorithmConfigurationForm) ) {
	      return new HAL.singleAlgorithmConfigurationForm(options);
	}
	
	var _idPrefix = options.idPrefix == null ? 'newSingleAlgorithmConfiguration' : options.idPrefix;
	
	var _newSingleAlgorithmConfigurationForm = $('<form></form>');
	_newSingleAlgorithmConfigurationForm.attr('id', _idPrefix+'_form');
	
	// we had to move specific onclick definitions here
	// because depending on whether we wanted to clone or create
	// we have to inject defaults, and this must happen
	// at the time that we create these functions.
	
	
	var _newCreateOrCloneMenu = new HAL.createOrCloneMenu({
		idPrefix : _idPrefix,
		enabled : true,
		expanded : true,
		showPrevious : false,
		showNext : false
	}).addNewButtonOnClickHandler(function() {
		
		_newCreateOrCloneMenu.collapse().disable();
		
		_experimentInformationMenu.enable().expand().clear();
		_experimentInformationMenu.getMenuObject().hidePreviousButton();
		
		_featuresMenu.addOnNext(function() {
			_featuresMenu.disable();
			_targetAlgorithmMenu.enable();
			_targetAlgorithmMenu.expand();
			
			_targetAlgorithmMenu.update(
				{
					'compatibleWithInstanceDistribution' : _experimentInformationMenu.getInstanceDistribution()	
				},
				function() {},
				function() {}
			);
		});
		
		_experimentInformationMenu.addOnPrevious(function() {
			_newCreateOrCloneMenu.enable();
			_newCreateOrCloneMenu.expand();
			_experimentInformationMenu.disable();
			_experimentInformationMenu.collapse();
		});
		
		_experimentInformationMenu.addOnNext(function() {
			if (_experimentInformationMenu.validate()) {
				_experimentInformationMenu.disable();
				
				var regex = new RegExp(/.*SMAC.*/);
				
				if (regex.exec(_experimentInformationMenu.getMetaAlgorithmicProcedure()) !== null) {
					_featuresMenu.enable();
					_featuresMenu.expand();
					
					_featuresMenu.update(
						{
							'compatibleWithInstanceDistribution' : _experimentInformationMenu.getInstanceDistribution()
						},
						function() {},
						function() {}
					);
				} else {
					_targetAlgorithmMenu.enable();
					_targetAlgorithmMenu.expand();
					
					_targetAlgorithmMenu.update(
						{
							'compatibleWithInstanceDistribution' : _experimentInformationMenu.getInstanceDistribution()	
						},
						function() {},
						function() {}
					);
				}
			}
		});
		
		_targetAlgorithmMenu.addOnNext(function() {
			if (_targetAlgorithmMenu.getAlgorithm(0) !== '') {
				_targetAlgorithmMenu.disable();
				_parametersMenu.enable();
				_parametersMenu.expand();
				
				_parametersMenu.update(
					{
						sourceAlgorithm : _experimentInformationMenu.getMetaAlgorithmicProcedure()
					},
					function() {},
					function() {}
				);
			}
		});
		
	}).addCloneButtonOnClickHandler(function() {
		_newCreateOrCloneMenu.disable();
		_cloneExperimentMenu.enable();
		_cloneExperimentMenu.expand();
		
	});
	
	var _cloneExperimentNextFunction = function() {
		//_newCreateOrCloneMenu.disable();
		
		_experimentInformationMenu.enable();
		_experimentInformationMenu.expand().isLoading();
		
		//_experimentInformationMenu.get
		
		$.ajax({
			url: '/jsp/getAlgorithmRunRequestJSON.jsp',
			data : { runId : _cloneExperimentMenu.getSelectorValue() },
			dataType: 'json',
			success: function(data) {
				if (data.successful) {
					var experimentObj = createExperimentObject(data.result);
					
					_experimentInformationMenu.hasLoaded().setDefaults({
						experimentName : experimentObj.name(),
						experimentDescription : experimentObj.description(),
						metaAlgorithm: experimentObj.metaAlgorithm(),
						instanceDistribution: experimentObj.instanceDistribution()
					});
					
					_executionEnvironmentMenu.setDefaults({
						environment: experimentObj.environment(),
					});
					
					submitButton.attr('disabled',false);
					
					// override addOnNext function so that
					// we can update with defaults
					_featuresMenu.addOnNext(function() {
						_featuresMenu.disable();
						_targetAlgorithmMenu.enable();
						_targetAlgorithmMenu.expand();
						
						_targetAlgorithmMenu.update(
							{
								'compatibleWithInstanceDistribution' : _experimentInformationMenu.getInstanceDistribution()	
							},
							function() { _targetAlgorithmMenu.setDefaults({ targetAlgorithms : experimentObj.targetAlgorithm() }); },
							function() {}
						);
					});
					
					_experimentInformationMenu.addOnPrevious(function() {
						_cloneExperimentMenu.enable().expand();
						_experimentInformationMenu.disable();
						_experimentInformationMenu.collapse();
					});
					
					_experimentInformationMenu.addOnNext(function() {
						if (_experimentInformationMenu.validate()) {
							_experimentInformationMenu.disable();
							
							var regex = new RegExp(/.*SMAC.*/);
							
							if (regex.exec(_experimentInformationMenu.getMetaAlgorithmicProcedure()) !== null) {
								_featuresMenu.enable();
								_featuresMenu.expand();
								
								_featuresMenu.update(
									{
										'compatibleWithInstanceDistribution' : _experimentInformationMenu.getInstanceDistribution()
									},
									function() {},
									function() {}
								);
							} else {
								_targetAlgorithmMenu.enable();
								_targetAlgorithmMenu.expand();
								
								_targetAlgorithmMenu.update(
									{
										'compatibleWithInstanceDistribution' : _experimentInformationMenu.getInstanceDistribution()	
									},
									function() { _targetAlgorithmMenu.setDefaults({ targetAlgorithms: experimentObj.targetAlgorithm() });
									},
									
									function() {}
								);
							}
						}
					});
					
					_targetAlgorithmMenu.addOnNext(function() {
						if (_targetAlgorithmMenu.getAlgorithm(0) !== '') {
							_targetAlgorithmMenu.disable();
							_parametersMenu.enable();
							_parametersMenu.expand();
							
							_parametersMenu.update(
								{
									sourceAlgorithm : _experimentInformationMenu.getMetaAlgorithmicProcedure()
								},
								function() { _parametersMenu.setDefaults({ metric: experimentObj.metric() }); },
								function() {},
								{
									scenario : experimentObj.scenarioOptions(),
									instance : experimentObj.instanceOptions(),
									configuration : experimentObj.configurationOptions(),
								}
							);
						}
					});
					
					
				}
			}
		});
	}
	
	var _cloneExperimentMenu = new HAL.cloneExperimentSelectionMenu({
		idPrefix : _idPrefix,
		//showPrevious : true,
		showPrevious : false,
		showNext : true,
		expanded : false,
		enabled : false
	}).addOnPrevious(function() {
		_cloneExperimentMenu.collapse().disable();
		_newCreateOrCloneMenu.enable().expand();
		
		
	}).addOnNext(function () {
		_cloneExperimentMenu.collapse().disable();
		_cloneExperimentNextFunction();
	});
	
	
	var _experimentInformationMenu = new HAL.defaultExperimentInformationMenu({
		idPrefix : _idPrefix,
		metaAlgorithmicPluginCompatibilityFilter : 'SingleAlgorithmConfiguration',
		showPrevious : true,
		//showPrevious : false,
		showNext : true,
		expanded : false,
		enabled : false,
		defaults : (!options.defaults) ? undefined : {
			experimentName : 		options.defaults.name(),
			experimentDescription: 	options.defaults.description(),
			metaAlgorithm :			options.defaults.metaAlgorithm(),
			instanceDistribution:	options.defaults.instanceDistribution()
		}
		
	});
		
	
	//new HAL.defaultExperimentInformationMenu(defaultExperimentInformationMenuOptions);
	
	var _featuresMenu = new HAL.defaultFeatureExtractionAndSelectionMenu({
		idPrefix : _idPrefix,
		enabled : false,
		expanded : false
	});
	
	
	//alert(options.defaults.targetAlgorithm() != undefined);
	//alert(JSON.stringify(options.defaults.targetAlgorithm()));
	
	
	var _targetAlgorithmMenu = new HAL.defaultMultipleAlgorithmSelectionMenu({
		idPrefix : _idPrefix,
		enabled : false,
		expanded : false,
		minimumAlgorithms : 1,
		maximumAlgorithms : 1,
		includeParameterisedAlgorithms : true,
		includeParameterlessAlgorithms : false,
		forceParameterlessSelection : false,
//		defaults: undefined
		defaults : (!options.defaults) ? undefined : {
			initialValues : options.defaults.targetAlgorithm(),
		}
	});
	
	var _parametersMenu = new HAL.defaultMetaAlgorithmParametersMenu({
		idPrefix : _idPrefix,
		headerContent : 'Design procedure parameters',
		enabled : false,
		expanded : false,
		defaults : (!options.defaults) ? undefined : {
			metric : options.defaults.metric(),
			scenario : options.defaults.scenarioOptions(),
			instance : options.defaults.instanceOptions(),
			configuration : options.defaults.configurationOptions(),
		}
	});
	
	var _executionEnvironmentMenu = new HAL.defaultExecutionEnvironmentMenu({
		idPrefix : _idPrefix,
		enabled : false,
		expanded : false,
		showNext : false,
		defaults : (!options.defaults) ? undefined : {
			environment : options.defaults.environment(),
		}
	});
	
	var _menuItems = [_experimentInformationMenu, _featuresMenu, _targetAlgorithmMenu, _parametersMenu, _executionEnvironmentMenu];
		
//	_experimentInformationMenu.addOnNext(function() {
//		if (_experimentInformationMenu.validate()) {
//			_experimentInformationMenu.disable();
//			
//			var regex = new RegExp(/.*SMAC.*/);
//			
//			if (regex.exec(_experimentInformationMenu.getMetaAlgorithmicProcedure()) !== null) {
//				_featuresMenu.enable();
//				_featuresMenu.expand();
//				
//				_featuresMenu.update(
//					{
//						'compatibleWithInstanceDistribution' : _experimentInformationMenu.getInstanceDistribution()
//					},
//					function() {},
//					function() {}
//				);
//			} else {
//				_targetAlgorithmMenu.enable();
//				_targetAlgorithmMenu.expand();
//				
//				_targetAlgorithmMenu.update(
//					{
//						'compatibleWithInstanceDistribution' : _experimentInformationMenu.getInstanceDistribution()	
//					},
//					function() {},
//					function() {}
//				);
//			}
//		}
//	});
	
//	_experimentInformationMenu.addOnPrevious(function() {
//		_newCreateOrCloneMenu.enable();
//		_newCreateOrCloneMenu.expand();
//		_experimentInformationMenu.disable();
//		_experimentInformationMenu.collapse();
//	});
	
	_featuresMenu.addOnPrevious(function() {
		_featuresMenu.disable();
		
		_experimentInformationMenu.enable();
		_experimentInformationMenu.expand();
	});
		
//	_featuresMenu.addOnNext(function() {
//		_featuresMenu.disable();
//		_targetAlgorithmMenu.enable();
//		_targetAlgorithmMenu.expand();
//		
//		_targetAlgorithmMenu.update(
//			{
//				'compatibleWithInstanceDistribution' : _experimentInformationMenu.getInstanceDistribution()	
//			},
//			function() {},
//			function() {}
//		);
//	});
	
	_targetAlgorithmMenu.addOnPrevious(function() {
		_targetAlgorithmMenu.disable();
		
		var regex = new RegExp(/.*SMAC.*/);
		
		if (regex.exec(_experimentInformationMenu.getMetaAlgorithmicProcedure()) !== null) {
			_featuresMenu.enable();
			_featuresMenu.expand();
		} else {
			_experimentInformationMenu.enable();
			_experimentInformationMenu.expand();
		}
	});
	
//	_targetAlgorithmMenu.addOnNext(function() {
//		if (_targetAlgorithmMenu.getAlgorithm(0) !== '') {
//			_targetAlgorithmMenu.disable();
//			_parametersMenu.enable();
//			_parametersMenu.expand();
//			
//			_parametersMenu.update(
//				{
//					sourceAlgorithm : _experimentInformationMenu.getMetaAlgorithmicProcedure()
//				},
//				function() {},
//				function() {}
//			);
//		}
//	});
	
	_parametersMenu.addOnPrevious(function() {
		_parametersMenu.disable();
		_targetAlgorithmMenu.enable();
		_targetAlgorithmMenu.expand();
	});
	
	_parametersMenu.addOnNext(function() {
		_parametersMenu.disable();
		_executionEnvironmentMenu.enable();
		_executionEnvironmentMenu.expand();
	});
	
	_executionEnvironmentMenu.addOnPrevious(function() {
		_executionEnvironmentMenu.clearMessages();
		_executionEnvironmentMenu.disable();
		
		_parametersMenu.enable();
		_parametersMenu.expand();
	});
	
	
	
	var submitButton = $('<input></input>');
	submitButton.attr('type', 'button');
	submitButton.attr('value', 'Submit');
	submitButton.attr('disabled', (options.defaults) ? false : true);
	submitButton.click(function() {
		_executionEnvironmentMenu.clearMessages();
		submitButton.attr('disabled', true);
		
		var formData = _newSingleAlgorithmConfigurationForm.serialize();
		formData = formData+'&hal_httpResponseContentType=json';
		
		$.post('/hal/processFormSnippet/createSingleAlgorithmConfiguration/createSingleAlgorithmConfiguration',
			formData,
			function(response) {
				submitButton.attr('disabled', false);
				
				if (response.successful) {
					_executionEnvironmentMenu.addSuccessMessage('Successfully created single algorithm configuration experiment!');
				} else {
					for (var i in response.errorMessages) {
						_executionEnvironmentMenu.addErrorMessage(response.errorMessages[i]);
					}
				}
			},
			'json'
		);
	});
	
	_executionEnvironmentMenu.addNavigationElement(submitButton);
	_executionEnvironmentMenu.addEnvironmentChangeCallback(function() {
		if (_executionEnvironmentMenu.getEnvironment() !== '') {
			submitButton.attr('disabled', false);
		} else {
			submitButton.attr('disabled', true);
		}
	});
	
	// Add the menu to the DOM.
	_newSingleAlgorithmConfigurationForm.append(_newCreateOrCloneMenu.getMenuElement());
	_newSingleAlgorithmConfigurationForm.append(_cloneExperimentMenu.getMenuElement());
	_newSingleAlgorithmConfigurationForm.append(_experimentInformationMenu.getMenuElement());
	_newSingleAlgorithmConfigurationForm.append(_featuresMenu.getMenuElement());
	_newSingleAlgorithmConfigurationForm.append(_targetAlgorithmMenu.getMenuElement());
	_newSingleAlgorithmConfigurationForm.append(_parametersMenu.getMenuElement());
	_newSingleAlgorithmConfigurationForm.append(_executionEnvironmentMenu.getMenuElement());
	
	return {
		getElement : function() {
			return _newSingleAlgorithmConfigurationForm;
		},
	};
};

HAL.createOrCloneMenu = function(options) {
	if ( !(this instanceof HAL.createOrCloneMenu) ) {
	      return new HAL.createOrCloneMenu(options);
	}
	
	var _headerContent = options.headerContent == null ? 'Experiment creation mode' : options.headerContent;
	var _idPrefix = options.idPrefix == null ? 'noid' : options.idPrefix;
	var _isMenuEnabled = options.enabled == null ? true : options.enabled;
	var _isMenuExpanded = options.expanded == null ? true : options.expanded;
	var _showPrevious = options.showPrevious == null ? true : options.showPrevious;
	var _showNext = options.showNext == null ? true : options.showNext;
	
	var _menu = new HAL.menu({
		header : _headerContent,
		enabled : _isMenuEnabled,
		expanded : _isMenuExpanded,
		showPrevious : _showPrevious,
		showNext : _showNext
	});

	// experiment information setup and navigation
	
	var description = $('<div></div>').addClass('formFieldDescription').html('Define a brand new configuration experiment or clone an already defined experiment');
	
	var newExperimentButton = $('<input></input>').attr('type','button').attr('value','Define new experiment');
	
	var cloneExperimentButton = $('<input></input>').attr('type','button').attr('value','Clone an existing experiment');
	
	_menu.getContentElement().append(description).append(newExperimentButton).append(cloneExperimentButton);
	
	return {
		getMenuElement : function() {
			return _menu.getMenuElement();
		},
		getMenuObject : function() {
			return _menu;
		},
		getContentElement : function() {
			return _menu.getContentElement();
		},
		addNewButtonOnClickHandler : function(f) {
			newExperimentButton.click(f);
			return this;
		},
		addCloneButtonOnClickHandler : function(f) {
			cloneExperimentButton.click(f);
			return this;
		},
		addOnNext : function(f) {
			_menu.addOnNext(f);
			return this;
		},
		addOnPrevious : function(f) {
			_menu.addOnPrevious(f);
			return this;
		},
		clear : function() {
			return this;
		},
		disable : function() {
			_menu.disable();
			return this;
		},
		enable : function() {
			_menu.enable();
			return this;
		},
		expand : function() {
			_menu.expand();
			return this;
		},
		collapse : function() {
			_menu.collapse();
			return this;
		}
	};
};

HAL.cloneExperimentSelectionMenu = function(options) {
	if ( !(this instanceof HAL.cloneExperimentSelectionMenu) ) {
	      return new HAL.cloneExperimentSelectionMenu(options);
	}
	
	var _headerContent = options.headerContent == null ? 'Clone experiment mode' : options.headerContent;
	var _idPrefix = options.idPrefix == null ? 'noid' : options.idPrefix;
	var _isMenuEnabled = options.enabled == null ? true : options.enabled;
	var _isMenuExpanded = options.expanded == null ? true : options.expanded;
	var _showPrevious = options.showPrevious == null ? true : options.showPrevious;
	var _showNext = options.showNext == null ? true : options.showNext;
	
	var _menu = new HAL.menu({
		header : _headerContent,
		enabled : _isMenuEnabled,
		expanded : _isMenuExpanded,
		showPrevious : _showPrevious,
		showNext : _showNext
	});

	// experiment information setup and navigation
	
	var description = $('<div></div>').addClass('formFieldDescription').html('Choose an experiment to clone.');
	
	var spinnerDiv = $('<div></div>').addClass('spinner').html($('<img></img')
			.attr('src', '/hal/commands/getFile/web/icons/ajax-loader.gif').attr('alt', 'Loading. Please wait.'));
	
	var selector = $('<select></select>');
	
	//var nullOption = $('<option></option>').html('Loading ...').attr('value','').appendTo(selector);
	
	$.ajax({
		url: '/jsp/getAlgorithmRunNames.jsp',
		dataType: 'json',
		success: function(data) {
			if (!data.successful) {
				// some error message
			} else {
				
				//nullOption.remove();
				
				spinnerDiv.remove();
				
				//nullOption = $('<option></option>').html('').attr('value','').appendTo(selector);
				
				
				for(var i in data.result) {
					var experiment = data.result[i];
					$('<option></option>').html(experiment.name).attr('value',experiment.id).appendTo(selector);
					
				}
				
				_menu.getContentElement().append(selector);
				
				
			}
		}
	});
	
	_menu.getContentElement().append(description).append(spinnerDiv);
	
	return {
		getMenuElement : function() {
			return _menu.getMenuElement();
		},
		getMenuObject : function() {
			return _menu;
		},
		getContentElement : function() {
			return _menu.getContentElement();
		},
		addSelectorOnClick : function(f) {
			selector.click(f);
			return this;
		},
		getSelectorValue : function() {
			return selector.val();
		},
		clear : function() {
			selector.val('');
			return this;
		},
		addOnNext : function(f) {
			_menu.addOnNext(f);
			return this;
		},
		addOnPrevious : function(f) {
			_menu.addOnPrevious(f);
			return this;
		},
		disable : function() {
			_menu.disable();
			return this;
		},
		enable : function() {
			_menu.enable();
			return this;
		},
		expand : function() {
			_menu.expand();
			return this;
		},
		collapse : function() {
			_menu.collapse();
			return this;
		}
	};
};

