function [timeToLearn, timeToPredict, results] = do_matrix_exp(func, matrix, Theta, all_features, names, options, seed, plotMatrices, tuningScenario, nTrain, nTest)
rand('twister',seed);
% Theta = Theta(1:11,:);
% all_features = all_features(1:10,:);
% matrix = matrix(1:10,1:10);
[C,I] = size(matrix); % C: #parameter configurations; I: #instances

%=== Construct data: all configs for 1st instance, all for 2nd, etc
nThetaTrain = ceil(C/2);
nInstTrain = ceil(I/2);

[cand_theta_idxs, cand_inst_idxs] = get_idx_pairs(1:nThetaTrain, 1:nInstTrain);
perm = randperm(length(cand_theta_idxs));
actual_nTrain = min(length(perm), nTrain);
train_idx = perm(1:actual_nTrain);

%=== Build X and y with light memory footprint.
[Xtrain,ytrain] = getXyForIndices(Theta, all_features, matrix, cand_theta_idxs(train_idx), cand_inst_idxs(train_idx));


%=== Assert we're treating categoricals right. 
for i=1:length(func.cat)
    for j=1:size(Xtrain,1)
        assert(check_member(Xtrain(j,i), func.all_values{i}));
        assert(Xtrain(j,i)>=1);
    end
end

fprintf(['Learning model of type ', options.modelType, ' ... \n']);
tic;
model = learnModel(Xtrain, ytrain, zeros(length(train_idx),1), func.cat, func.all_values, 0, options, names);
timeToLearn = toc
results = {};

plotMatrices=1;
if plotMatrices
    fprintf(['Doing predictions ...\n']);
    [all_theta_idxs, all_inst_idxs] = get_idx_pairs(1:C, 1:I);

    %=== Light memory footprint for predictions: do it in chunks.
    chunk_size = ceil(1000000/(size(Theta,2)+size(all_features,2)));
    
    ytest = -ones(length(all_theta_idxs),1);
    start_idx = 1;
    tic;
    while start_idx <= length(all_inst_idxs)
        fprintf(['Chunk %d/%d ...\n'], ceil(start_idx/chunk_size), ceil(length(all_inst_idxs)/chunk_size));
        end_idx = min(start_idx+chunk_size, length(all_inst_idxs));
        
        [Xtest,ytest(start_idx:end_idx,1)] = getXyForIndices(Theta, all_features, matrix, all_theta_idxs(start_idx:end_idx), all_inst_idxs(start_idx:end_idx));
        [ypred(start_idx:end_idx,1), ypredvar(start_idx:end_idx,1)] = applyModel(model, Xtest, 0, 0, 0);
        start_idx = end_idx+1;
    end
    timeToPredict = toc

    [rmse, ll, cc, cc_rank] = measures_of_fit(log10(ytest), ypred, ypredvar);
    fprintf('Overall: RMSE = %f, LL=%f, CC=%f, CC_rank=%f\n', [rmse, ll, cc, cc_rank]);

%    fullpredmatrix = 10.^(reshape(ypred, [C, I]));
%    fullpredmatrixvar = reshape(ypredvar, [C, I]);
%    assert(all(all( abs(matrix - reshape(ytest, [C, I]))<0.000001 ))); % assert that we're reshaping right.
    fullpredmatrix = 10.^(reshape(ypred, [I, C])');
    fullpredmatrixvar = reshape(ypredvar, [I, C])';
    assert(all(all( abs(matrix - reshape(ytest, [I, C])')<0.000001 ))); % assert that we're reshaping right.

    %=== Compute goodness of each configuration.
    inst_hardness = -ones(1,I);
    for i=1:I
         inst_hardness(i) = combine_performance(matrix(:,i));
    end
    [tmp,ind1] = sort(inst_hardness);

    %=== Compute goodness of each configuration.
    allscores = -ones(1,C);
    for i=1:C
         allscores(i) = combine_performance(matrix(i,:)');
    end
    [tmp,ind2] = sort(allscores);

    % 4 subplots
    learnTime = timeToLearn;
    % subplot(2,2,1);
    results{end+1} = predict_submatrix(matrix, fullpredmatrix, fullpredmatrixvar, ind1(find(ind1 <= nInstTrain)), ind2(find(ind2 <= nThetaTrain)), tuningScenario, '-trainC_trainI', options.modelType, learnTime);

    %subplot(2,2,2);
    results{end+1} = predict_submatrix(matrix, fullpredmatrix, fullpredmatrixvar, ind1(find(ind1 <= nInstTrain)), ind2(find(ind2 > nThetaTrain)), tuningScenario, '-testC_trainI', options.modelType, learnTime);

    %subplot(2,2,3);
    results{end+1} = predict_submatrix(matrix, fullpredmatrix, fullpredmatrixvar, ind1(find(ind1 >  nInstTrain)), ind2(find(ind2 <= nThetaTrain)), tuningScenario, '-trainC_testI', options.modelType, learnTime);

    %subplot(2,2,4);
    results{end+1} = predict_submatrix(matrix, fullpredmatrix, fullpredmatrixvar, ind1(find(ind1 > nInstTrain)), ind2(find(ind2 > nThetaTrain)), tuningScenario, '-testC_testI', options.modelType, learnTime);
else
    %=== 1st quadrant: if we have enough data to have train and test sets be disjoint, then do that.
    tic;
    actual_nTest = min(length(perm), nTest);
    test_idx = perm(end-actual_nTest+1:end); % overlaps with train for small data set
    [Xtest,ytest] = getXyForIndices(Theta, all_features, matrix, cand_theta_idxs(test_idx), cand_inst_idxs(test_idx));
    [ypred, ypredvar] = applyModel(model, Xtest, 0, 0, 0);
    [rmse, ll, cc, cc_rank] = measures_of_fit(log10(ytest), ypred, ypredvar);
    results{end+1} = [rmse, ll, cc_rank];
    
    %=== 2nd, 3rd, and 4th quadrant: guaranteed not to overlap with training set.
    [rmse, ll, cc_rank] = get_results_for_quadrant( model, Theta, all_features, matrix, nTest, 1:nThetaTrain, nInstTrain+1:I );
    results{end+1} = [rmse, ll, cc_rank];
    [rmse, ll, cc_rank] = get_results_for_quadrant( model, Theta, all_features, matrix, nTest, nThetaTrain+1:C, 1:nInstTrain );
    results{end+1} = [rmse, ll, cc_rank];
    [rmse, ll, cc_rank] = get_results_for_quadrant( model, Theta, all_features, matrix, nTest, nThetaTrain+1:C, nInstTrain+1:I );
    results{end+1} = [rmse, ll, cc_rank];
    timeToPredict = toc
end

function [rmse, ll, cc_rank] = get_results_for_quadrant( model, Theta, all_features, matrix, nTest, cand_theta_idxs, cand_inst_idxs )
perm = randperm(length(cand_theta_idxs));
actual_nTest = min(length(perm), nTest);
test_idx = perm(end-actual_nTest+1:end);
[Xtest,ytest] = getXyForIndices(Theta, all_features, matrix, cand_theta_idxs(test_idx), cand_inst_idxs(test_idx));
[ypred, ypredvar] = applyModel(model, Xtest, 0, 0, 0);
[rmse, ll, cc, cc_rank] = measures_of_fit(log10(ytest), ypred, ypredvar);

% function [X,y] = sample_Xy_for_cand_idxs(Theta, all_features, matrix, nTest, theta_idx_candidates, inst_idx_candidates)
% [cand_theta_idxs, cand_inst_idxs] = get_idx_pairs(theta_idx_candidates, inst_idx_candidates)
% perm = randperm(1:length(cand_theta_idxs));
% perm = perm(1:nTest);
% theta_idxs = cand_theta_idxs(perm);
% inst_idxs = cand_inst_idxs(perm);
% [X,y] = getXyForIndices(Theta, all_features, matrix, theta_idxs, inst_idxs);



function [res] = predict_submatrix(matrix, fullpredmatrix, fullpredmatrixvar, ind1, ind2, tuningScenario, suffix, modelType, learnTime)
truematrix = matrix(ind2, ind1);
thispredmatrix = fullpredmatrix(ind2, ind1);
thispredmatrixvar = fullpredmatrixvar(ind2, ind1);

[rmse, ll, cc, cc_rank] = measures_of_fit(log10(truematrix(:)), log10(thispredmatrix(:)), thispredmatrixvar(:));
fprintf(strcat([modelType, ' performance for ', suffix, ': RMSE = %f, LL=%f, CC_rank=%f, learnTime=%f\n']), [rmse, ll, cc_rank, learnTime]); % CC=%f, cc, 

figure
h=image(log10(thispredmatrix), 'CDataMapping', 'scaled'); 
colormap(gray); 
hold on; 
%title('mean pred matrix'); 
ylabel('config (sorted by true goodness)', 'FontSize', 22); 
xlabel('instance (sorted by true hardness)', 'FontSize', 22);
h=colorbar;
mini = min(min(log10(thispredmatrix)));
maxi = max(max(log10(thispredmatrix)));
set(gca, 'FontSize', 16);
set(h, 'YLim', [mini,maxi])
filename_prefix = strcat([tuningScenario, '-predmatrix', suffix, '-', modelType]);
saveas(gcf, strcat(filename_prefix,'.fig'));
set(gcf, 'PaperPositionMode', 'auto');
pdf_filename = strcat(filename_prefix,'.pdf');
if exist(pdf_filename, 'file');
    delete(pdf_filename);
end
export_fig(pdf_filename);

figure
h=image(log10(truematrix), 'CDataMapping', 'scaled'); 
colormap(gray); 
hold on; 
%    title(tuningScenario); 
ylabel('config (sorted by true goodness)', 'FontSize', 22); 
xlabel('instance (sorted by true hardness)', 'FontSize', 22);
h=colorbar;
mini = min(min(log10(truematrix)));
maxi = max(max(log10(truematrix)));
set(gca, 'FontSize', 16);
set(h, 'YLim', [mini,maxi])
filename_prefix = strcat([tuningScenario, '-truematrix', suffix]);
saveas(gcf, strcat(filename_prefix,'.fig'));
set(gcf, 'PaperPositionMode', 'auto');
pdf_filename = strcat(filename_prefix,'.pdf');
if exist(pdf_filename, 'file');
    delete(pdf_filename);
end
export_fig(pdf_filename);
res = [rmse, ll, cc_rank];

% figure
% trueMarginalMean = mean(truematrix, 2);
% predMarginalMean = mean(thispredmatrix, 2);
% plot(log10(trueMarginalMean), log10(predMarginalMean), '.');
% 
% [rmse, ll, cc, cc_rank] = measures_of_fit(log10(trueMarginalMean(:)), log10(predMarginalMean(:)), zeros(size(predMarginalMean(:),1),1));
% rmse
   
function a=combine_performance(scores)
% Other performance metrics, such as median etc could be implemented here.
a=mean(scores,1);