function plot_Scaling_N(numTrains, data_avg, data_std, options_vec, dataname, model_names, domain, legendPos, y_log)
if nargin < 9
    y_log = 0;
    if nargin < 8
        legendPos = 'SouthEast';
    end
end

legend_entries = {};
hE = [];
figure
hold off
if y_log
    h = loglog(numTrains, data_avg(:,1));
else
    h = semilogx(numTrains, data_avg(:,1));
end
hold on
for model_idx=1:length(options_vec)
    hE(end+1) = errorbar(numTrains*(1.05^(model_idx-1)), data_avg(:,model_idx), data_std(:,model_idx), 'r');
    legend_entries{end+1} = model_names(model_idx);
    hXLabel = xlabel('Number of training data points, n');
    hYLabel = ylabel(dataname);
end

mintime = min(numTrains)/1.3;
maxtime = max(numTrains)*1.3;

set(h, 'Color' , [1 1 1]);
set([hXLabel, hYLabel], 'FontSize', 16);

set(hE(1)                      , ...
  'LineStyle'       , ':'      , ...
  'LineWidth'       , 2           , ...
  'Color'           , [.4 .4 .4]  , ...
  'MarkerSize'      , 6           , ...
  'Marker'          , 'o'         , ...
  'MarkerFaceColor' , [.7 .7 .7]  , ...
  'MarkerEdgeColor' , [.4 .4 .4]);

if length(hE) > 1
    set([hE(2)]                            , ...
        'Color'           , 'b'  , ...
        'Marker'          , 'x'         , ...
        'MarkerSize'      , 8, ...        
        'LineWidth'       , 2);

    if length(hE) > 2
        set([hE(3)]                            , ...
            'Color'           , 'r'  , ...
            'Marker'          , '.'         , ...
            'LineStyle'       , '--'      , ...
            'MarkerSize'      , 16, ...        
            'LineWidth'       , 2);
    end
end

set(gca, ...
    'XTick'       , numTrains, ...
    'FontName'   , 'Helvetica' , ...
    'FontSize'   , 16);
%    'YTick'       , -1:0.1:1   , ...

max_data = max(max(data_avg+data_std + 0.03));
min_data = min(min(data_avg-data_std - 0.03));

axis([mintime, maxtime, min_data, max_data])

legend_str = legend_entries{1};
for i=2:length(legend_entries)
    legend_str = strcat([legend_str, legend_entries{i}]);
end
legend(hE, legend_str, 'location', legendPos);
dataname = regexprep(dataname, ' ', '_');
filename = strcat('scale_N_', domain, '_', dataname);
pdf_filename = strcat(filename, '.pdf');
if exist(pdf_filename, 'file');
    delete(pdf_filename);
end
export_fig(pdf_filename);    
saveas(gca, strcat(filename, '.fig'), 'fig');