package ca.ubc.cs.beta.hal.algorithms.metaalgorithms.design.paramils;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.util.LinkedList;
import java.util.logging.Level;
import java.util.logging.Logger;

import net.sf.json.JSONArray;

import org.apache.commons.io.FileUtils;
import org.junit.Test;

import ca.ubc.cs.beta.hal.algorithms.Algorithm;
import ca.ubc.cs.beta.hal.algorithms.AlgorithmRunRequest;
import ca.ubc.cs.beta.hal.algorithms.ParameterizedAlgorithm;
import ca.ubc.cs.beta.hal.algorithms.ParameterlessAlgorithm;
import ca.ubc.cs.beta.hal.algorithms.parameters.ParameterSettingBuilder;
import ca.ubc.cs.beta.hal.algorithms.parameters.Semantics;
import ca.ubc.cs.beta.hal.environments.AlgorithmRun;
import ca.ubc.cs.beta.hal.environments.AlgorithmRun.RunStatus;
import ca.ubc.cs.beta.hal.environments.Environment;
import ca.ubc.cs.beta.hal.environments.ExecutionManager.OutputHandlingOption;
import ca.ubc.cs.beta.hal.environments.datamanagers.SQLiteDataManager;
import ca.ubc.cs.beta.hal.environments.executionmanagers.LocalExecutionManager;
import ca.ubc.cs.beta.hal.problems.FileProblemInstance;
import ca.ubc.cs.beta.hal.problems.InstanceList;
import ca.ubc.cs.beta.hal.problems.InstanceMetricMetaProblemInstance;
import ca.ubc.cs.beta.hal.problems.MetaProblemInstance;
import ca.ubc.cs.beta.hal.problems.ProblemInstance;
import ca.ubc.cs.beta.hal.problems.metrics.PerformanceMetric;
import ca.ubc.cs.beta.hal.utils.Global;

public class ExternalParamILSImplementationTest {
	
	static {
		Logger.getLogger("ca.ubc.cs.beta.hal").setLevel(Level.WARNING);
//		Logger.getLogger("ca.ubc.cs.beta.hal.algorithms.metaalgorithms.design.externalconfigurators").setLevel(Level.ALL);
//		Logger.getLogger("ca.ubc.cs.beta.hal.environments.datamanagers").setLevel(Level.ALL);
	}
	
	@Test
	public void makeSpec() throws IOException {
		JSONArray a = new JSONArray();
		a.add(new ExternalParamILSImplementation().toSpec());
		FileUtils.writeStringToFile(new File("pils.json"), a.toString(2));
	}
	
	@Test
	public void testRuns() throws Exception {
			
		File dbfile = new File("test/pils.db");
		if (dbfile.exists()) dbfile.delete();
		SQLiteDataManager dm = new SQLiteDataManager(URI.create("jdbc:sqlite:test/pils.db"));
//		MySQLDataManager dm = new MySQLDataManager(URI.create("jdbc:mysql://localhost/pils_unittest"));
		dm.clear();
		Global.setDataManager(dm);
		
		LocalExecutionManager lem = new LocalExecutionManager(OutputHandlingOption.ECHO, OutputHandlingOption.ECHO);
		final Environment env = new Environment("local unittest", lem);
						
		// get the meta-algorithm
		ParameterizedAlgorithm configurator = new ExternalParamILSImplementation().getDefaultParameterizedAlgorithm();		
		String json = "{'command':'python', " +
				"'" + Semantics.CUTOFF_AGNOSTIC + "': false, " +
				"'" + Semantics.EXPORTABLE + "': false, "+
				"'name':'target', 'tags':[['foop']], 'outputFormat':{'stdout':['foroutput $v$ $t$ $q$']}, 'outputs':{'v':{'domain':'Real()', 'scale':'LOG'}, 't':{'domain':'Real()', 'semantics':'CPUTIME'}, 'q':{'domain':'Real()', 'semantics':'SOLUTION_QUALITY'}}, 'name':'json1', 'path':'test', 'deterministic':false, 'inputFormat':{'callstring':['test.py $time$ [$hitime$]']}, 'inputs':{'time':{'domain':'Real[0.001, 0.5]', 'scale':'LINEAR', 'default':0.25}, 'hitime':{'domain':'Real[0.5, 1]', 'default':0.75, 'scale':'LINEAR'}}}";
		ParameterizedAlgorithm alg1 = ParameterizedAlgorithm.fromSpec(json);
		
//		FileUtils.writeStringToFile(new File("pils.alg"), configurator.toSpec());
		
		PerformanceMetric metric = PerformanceMetric.PAR10;//MEDR;
//		PerformanceMetric metric = PerformanceMetric.ASQ;
		
		// create some dummy instances for our dummy algorithm
		LinkedList<ProblemInstance> il = new LinkedList<ProblemInstance>();
		for (int i=0; i<4; i++) {
			File f = File.createTempFile("dummy", ".inst", new File("."));
			f.deleteOnExit();
			FileUtils.writeStringToFile(f, f.getPath());
			il.add(new FileProblemInstance(f));
		}
		InstanceList instances = new InstanceList("dummylist", il);
		instances.addTagsToAllInstances("foop");
		ParameterSettingBuilder psb = new ParameterSettingBuilder();
		psb.put(Semantics.MAX_SUBRUN_CPUTIME, 1.0);
		MetaProblemInstance inst = new InstanceMetricMetaProblemInstance(alg1, instances, metric, "dummyConfigProb", psb.build());
//		FileUtils.writeStringToFile(new File("sample.designinst"), inst.toSpec());
				
		configurator.setProblemInstance(inst);
		configurator.setScenarioValue(Semantics.MAX_RUNLENGTH, 30);
		configurator.setScenarioValue(ExternalParamILSImplementation.TRAINING_SET_SIZE, 100);
		
		AlgorithmRunRequest req = configurator.getAlgorithmRunRequest();
		req.setName("test pils run");
		AlgorithmRun run = env.fetchRun(req);
		Global.getThreadPool().execute(run);
		run.waitForVisitors();
		assertTrue(RunStatus.isFinished(run.getStatus()));
		
		assertTrue(RunStatus.finishedWithoutError(run.getStatus()));
		
		ParameterlessAlgorithm design = (ParameterlessAlgorithm) run.getLastOutputValueOnly(Semantics.DESIGN);
		
		System.out.println(run.getOutput());
		System.out.println(inst.getAlgorithms());
		
		assertEquals(design.getImplementation(), ((Algorithm)inst.getAlgorithms()).getImplementation());
		System.out.println("CFG" + design.getConfiguration());
		
		System.out.println("total " + run.getTotalCpuTime());
		System.out.println("measured " + run.getMeasuredCpuTime());
		System.out.println("overhead " + run.getOverheadCpuTime());
		System.out.println("subs " + run.getSubrunTotalCpuTime());
		System.out.println("subM " + run.getSubrunMeasuredCpuTime());
		System.out.println("subO " + run.getSubrunOverheadCpuTime());
		
		System.out.println("number of subruns: " + run.getSubrunCount());
		
		Thread.sleep(10000);
		
		assertTrue(dm.getAlgorithmNames(alg1.getName(), alg1.getVersion()).contains("test pils run design"));

//		// validate
//		InstanceMetricMetaProblemInstance validationInst = new InstanceMetricMetaProblemInstance(
//				new ParameterlessAlgorithmList(design), instances, metric, 
//				"dummyValidationInstance");
//		validationInst.setOption(Semantics.MAX_SUBRUN_CPUTIME, 1.0);
////		validationInst.setOption(Semantics.RUNS_PER_INSTANCE, 10);
//		ParameterizedAlgorithm alg2 = new SingleAlgorithmAnalysisMetaAlgorithmImplementation().getDefaultParameterizedAlgorithm();
//		alg2.setProblemInstance(validationInst);
//		alg2.setScenarioValue(Semantics.MAX_RUNLENGTH, 20);
//		AlgorithmRun validationRun = env.fetchRun(alg2.getAlgorithmRunRequest());
//		Global.getThreadPool().execute(validationRun);
//		validationRun.waitFor();
//		
//		Plotter p = new GnuplotPlotter(GnuplotPlotter.DFLT_PROPS);
//		System.out.println(validationRun.getLastOutputValueOnly());
//		p.display((Plot)validationRun.getLastOutputValueOnly(SingleAlgorithmAnalysisMetaAlgorithmImplementation.SCD));
//		
		
		
//		// wait for everything to commit.
//		do {
//			Thread.sleep(200);
//		} while (dm.getActiveConnections() > 0);
//		
//		// should come from the DB
//		for (int i=0; i<10; i++) {
//			run = env.fetchRun(configurator.getAlgorithmRunRequest());
//			assertTrue(run instanceof DatabaseAlgorithmRun);
//		}
	}
	
	@Test
	public void testTerminationStatus() throws Exception {
			
		SQLiteDataManager dm = new SQLiteDataManager(URI.create("jdbc:sqlite:test/pils.db"));
		dm.clear();
		Global.setDataManager(dm);
		
		LocalExecutionManager lem = new LocalExecutionManager(OutputHandlingOption.ECHO, OutputHandlingOption.ECHO);
		final Environment env = new Environment("local unittest", lem);
						
		// get the meta-algorithm
		ParameterizedAlgorithm configurator = new ExternalParamILSImplementation().getDefaultParameterizedAlgorithm();
		
		String json = "{'command':'python', " +
				"'" + Semantics.CUTOFF_AGNOSTIC + "': false, " +
				"'" + Semantics.EXPORTABLE + "': false, "+
				"'name':'target', 'tags':[['foop']], 'outputFormat':{'stdout':['foroutput $v$ $t$']}, 'outputs':{'v':{'domain':'Real()'}, 't':{'domain':'Real()'}}, 'name':'json1', 'path':'test', 'deterministic':false, 'inputFormat':{'callstring':['test.py $time$ [$hitime$]']}, 'inputs':{'time':{'domain':'Real[0, 0.5]', 'scale':'EXP', 'default':0.25}, 'hitime':{'domain':'Real[0.5, 1]', 'default':0.75}}}";
		ParameterizedAlgorithm alg1 = ParameterizedAlgorithm.fromSpec(json);
		
//		FileUtils.writeStringToFile(new File("pils.alg"), configurator.toSpec());
		
		PerformanceMetric metric = PerformanceMetric.PAR1;
		
		// create some dummy instances for our dummy algorithm
		LinkedList<ProblemInstance> il = new LinkedList<ProblemInstance>();
		for (int i=0; i<4; i++) {
			File f = File.createTempFile("dummy", ".inst", new File("."));
			f.deleteOnExit();
			FileUtils.writeStringToFile(f, f.getPath());
			il.add(new FileProblemInstance(f));
		}
		InstanceList instances = new InstanceList("dummylist", il);
		instances.addTagsToAllInstances("foop");
		
		ParameterSettingBuilder psb = new ParameterSettingBuilder();
		psb.put(Semantics.MAX_SUBRUN_CPUTIME, 1.0);
		MetaProblemInstance inst = new InstanceMetricMetaProblemInstance(alg1, instances, metric, "dummyConfigProb", psb.build());
		
//		FileUtils.writeStringToFile(new File("sample.designinst"), inst.toSpec());
				
		configurator.setProblemInstance(inst);
		configurator.setScenarioValue(Semantics.MAX_RUNLENGTH, 10);
		configurator.setScenarioValue(ExternalParamILSImplementation.TRAINING_SET_SIZE, 100);
		
		AlgorithmRun run = env.fetchRun(configurator.getAlgorithmRunRequest());
		Global.getThreadPool().execute(run);
		
		Thread.sleep(5000);
		run.terminate(RunStatus.TIMEOUT);
		run.waitForVisitors();
		assertTrue(RunStatus.isFinished(run.getStatus()));		
		assertTrue(RunStatus.timedOut(run.getStatus()));
		
		ParameterlessAlgorithm design = (ParameterlessAlgorithm) run.getLastOutputValueOnly(Semantics.DESIGN);
//		
//		System.out.println(run.getOutput());
//		System.out.println(inst.getAlgorithms());
//		
		assertEquals(design.getImplementation(), ((Algorithm)inst.getAlgorithms()).getImplementation());
//		System.out.println("CFG" + design.getConfiguration());
//		
//		System.out.println("total time:" + run.getCpuTime());
//		System.out.println("configurator/overhead time: " + run.getOverheadCpuTime());
//		System.out.println("subrun time: " + run.getSubrunCpuTime());
//		
//		System.out.println("number of subruns: " + run.getSubruns().size());
	}
	
	@Test
	public void testOutputsMakeSense() throws Exception {
		
		SQLiteDataManager dm = new SQLiteDataManager(URI.create("jdbc:sqlite:test/pils.db"));
//		MySQLDataManager dm = new MySQLDataManager(URI.create("jdbc:mysql://localhost/pils_unittest"));
		dm.clear();
		Global.setDataManager(dm);
		
		LocalExecutionManager lem = new LocalExecutionManager(OutputHandlingOption.ECHO, OutputHandlingOption.ECHO);
		final Environment env = new Environment("local unittest", lem);
						
		// get the meta-algorithm
		ParameterizedAlgorithm configurator = new ExternalParamILSImplementation().getDefaultParameterizedAlgorithm();		
		String json = "{'command':'python', " +
				"'" + Semantics.CUTOFF_AGNOSTIC + "': false, " +
				"'" + Semantics.EXPORTABLE + "': false, "+
				"'name':'target', 'tags':[['foop']], 'outputFormat':{'stdout':['foroutput $v$ $t$ $q$']}, 'outputs':{'v':{'domain':'Real()', 'scale':'LOG'}, 't':{'domain':'Real()', 'semantics':'CPUTIME'}, 'q':{'domain':'Real()', 'semantics':'SOLUTION_QUALITY'}}, 'name':'json1', 'path':'test', 'deterministic':false, 'inputFormat':{'callstring':['test.py $time$ [$hitime$]']}, 'inputs':{'time':{'domain':'Real[1,2]', 'scale':'LINEAR', 'default':1.25}, 'hitime':{'domain':'Real[2, 3]', 'default':2.75, 'scale':'LINEAR'}}}";
		ParameterizedAlgorithm alg1 = ParameterizedAlgorithm.fromSpec(json);
		
		PerformanceMetric metric = PerformanceMetric.PAR10;
		
		// create some dummy instances for our dummy algorithm
		LinkedList<ProblemInstance> il = new LinkedList<ProblemInstance>();
		for (int i=0; i<4; i++) {
			File f = File.createTempFile("dummy", ".inst", new File("."));
			f.deleteOnExit();
			FileUtils.writeStringToFile(f, f.getPath());
			il.add(new FileProblemInstance(f));
		}
		InstanceList instances = new InstanceList("dummylist", il);
		instances.addTagsToAllInstances("foop");
		ParameterSettingBuilder psb = new ParameterSettingBuilder();
		psb.put(Semantics.MAX_SUBRUN_CPUTIME, 1.0); // => every run should be a timeout no matter what
		MetaProblemInstance inst = new InstanceMetricMetaProblemInstance(alg1, instances, metric, "dummyConfigProb", psb.build());
//		FileUtils.writeStringToFile(new File("sample.designinst"), inst.toSpec());
				
		configurator.setProblemInstance(inst);
		configurator.setScenarioValue(Semantics.MAX_RUNLENGTH, 1); // => just one run
		configurator.setScenarioValue(ExternalParamILSImplementation.TRAINING_SET_SIZE, 100);
		
		AlgorithmRunRequest req = configurator.getAlgorithmRunRequest();
		req.setName("test pils run");
		AlgorithmRun run = env.fetchRun(req);
		Global.getThreadPool().execute(run);
		run.waitForVisitors();
		assertTrue(RunStatus.isFinished(run.getStatus()));
		
		assertEquals(1L, run.getSubrunCount());
		assertEquals(10.0, ((Number)run.getLastOutputValueOnly(Semantics.SOLUTION_QUALITY)).doubleValue(), 0.001);
	}
}
