package ca.ubc.cs.beta.hal.algorithms.metaalgorithms.design.paramils.oneoffs;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.junit.Test;

import ca.ubc.cs.beta.hal.algorithms.ParameterizedAlgorithm;
import ca.ubc.cs.beta.hal.algorithms.metaalgorithms.design.externalconfigurators.AbstractExternalConfiguratorImplementation;
import ca.ubc.cs.beta.hal.algorithms.metaalgorithms.design.paramils.ExternalParamILSImplementation;
import ca.ubc.cs.beta.hal.algorithms.parameters.ParameterSettingBuilder;
import ca.ubc.cs.beta.hal.algorithms.parameters.Semantics;
import ca.ubc.cs.beta.hal.environments.AlgorithmRun;
import ca.ubc.cs.beta.hal.problems.InstanceList;
import ca.ubc.cs.beta.hal.problems.InstanceMetricMetaProblemInstance;
import ca.ubc.cs.beta.hal.problems.metrics.PerformanceMetric;


public class CPLEXBugTest {
	@Test
	public void testSAPSBug() throws Exception {
		ParameterizedAlgorithm saps = null;
		if (System.getProperty("os.name").contains("Mac OS X")) {
			saps = ParameterizedAlgorithm.fromSpec("test/ca/ubc/cs/beta/hal/algorithms/metaalgorithms/design/paramils/oneoffs/cplex-discrete.cfg");
		} else {
			System.err.println("Not supported on non-OSX operating systems");
			fail();
		}
		ParameterizedAlgorithm pils = new ExternalParamILSImplementation().getDefaultParameterizedAlgorithm();
		pils.setScenarioValue(AbstractExternalConfiguratorImplementation.TRAINING_SET_SIZE, 1000);
		pils.setScenarioValue(Semantics.MAX_CPUTIME, 1800);
		pils.setScenarioValue(Semantics.SEED, 0);
		
		PerformanceMetric metric = PerformanceMetric.PAR1;
		InstanceList insts = InstanceList.fromDirectory(new File("test/ca/ubc/cs/beta/hal/algorithms/metaalgorithms/design/paramils/oneoffs"), ".*[.]cnf", "medinst", false, "cnf");
		
		ParameterSettingBuilder psb = new ParameterSettingBuilder();
		psb.put(Semantics.MAX_SUBRUN_CPUTIME, 5.0);
		InstanceMetricMetaProblemInstance mpi = new InstanceMetricMetaProblemInstance(saps, insts, metric, "testbreakinst", psb.build());
		pils.setProblemInstance(mpi);
		
		AlgorithmRun r = ((ExternalParamILSImplementation)pils.getImplementation()).getRun(pils.getAlgorithmRunRequest(), 
				null, null, null);
		r.getAlgorithmRunRequest().setId(null, 1L);
		
		Method m = r.getClass().getSuperclass().getDeclaredMethod("start");
		m.setAccessible(true);
		try {
			m.invoke(r);
		} catch (InvocationTargetException e) {
			// it will fail, but not because of a bad maxEvals setting
			String message = e.getCause().getMessage();
			message = message == null ? "" : message;
			assertFalse(message.contains("specified for: maxEvals"));
			
			// it should actually fail with an npe
			assertTrue(e.getCause() instanceof NullPointerException);
		}
	}
}
