package ca.ubc.cs.beta.hal.algorithms.metaalgorithms.design.paramils.oneoffs;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.io.File;
import java.net.URI;
import java.util.logging.LogRecord;

import org.apache.commons.io.FileUtils;
import org.junit.Test;

import ca.ubc.cs.beta.hal.algorithms.Algorithm;
import ca.ubc.cs.beta.hal.algorithms.ParameterizedAlgorithm;
import ca.ubc.cs.beta.hal.algorithms.metaalgorithms.design.externalconfigurators.AbstractExternalConfiguratorImplementation;
import ca.ubc.cs.beta.hal.algorithms.metaalgorithms.design.paramils.ExternalParamILSImplementation;
import ca.ubc.cs.beta.hal.algorithms.parameters.ParameterSettingBuilder;
import ca.ubc.cs.beta.hal.algorithms.parameters.Semantics;
import ca.ubc.cs.beta.hal.environments.AlgorithmRun;
import ca.ubc.cs.beta.hal.environments.AlgorithmRun.RunStatus;
import ca.ubc.cs.beta.hal.environments.Environment;
import ca.ubc.cs.beta.hal.environments.ExecutionManager.OutputHandlingOption;
import ca.ubc.cs.beta.hal.environments.datamanagers.SQLiteDataManager;
import ca.ubc.cs.beta.hal.environments.executionmanagers.LocalExecutionManager;
import ca.ubc.cs.beta.hal.problems.InstanceList;
import ca.ubc.cs.beta.hal.problems.InstanceMetricMetaProblemInstance;
import ca.ubc.cs.beta.hal.problems.metrics.PerformanceMetric;
import ca.ubc.cs.beta.hal.utils.Global;
import ca.ubc.cs.beta.hal.utils.Pair;


public class SAPSBugTest {
	@Test
	public void testSAPSBug() throws Exception {
		ParameterizedAlgorithm saps = null;
		if (System.getProperty("os.name").contains("Mac OS X")) {
			saps = ParameterizedAlgorithm.fromSpec("test/ca/ubc/cs/beta/hal/algorithms/metaalgorithms/design/paramils/oneoffs/saps-discreteMac.cfg");
		} else {
			System.err.println("Not supported on non-OSX operating systems");
			fail();
		}
		ParameterizedAlgorithm pils = new ExternalParamILSImplementation().getDefaultParameterizedAlgorithm();
		pils.setScenarioValue(AbstractExternalConfiguratorImplementation.TRAINING_SET_SIZE, 1000);
		pils.setScenarioValue(Semantics.MAX_CPUTIME, 1800);
		pils.setScenarioValue(Semantics.SEED, 0);
		
		PerformanceMetric metric = PerformanceMetric.PAR1;
		InstanceList insts = InstanceList.fromDirectory(new File("test/ca/ubc/cs/beta/hal/algorithms/metaalgorithms/design/paramils/oneoffs"), ".*[.]cnf", "medinst", false, "cnf");
		
		ParameterSettingBuilder psb = new ParameterSettingBuilder();
		psb.put(Semantics.MAX_SUBRUN_CPUTIME, 5.0);
		InstanceMetricMetaProblemInstance mpi = new InstanceMetricMetaProblemInstance(saps, insts, metric, "testbreakinst", psb.build());
		pils.setProblemInstance(mpi);
		
		FileUtils.writeStringToFile(new File("sapstest.metainst"), mpi.toSpec());
		
		File f = File.createTempFile("saps", ".testdb", new File("."));
		f.deleteOnExit();
		Global.setDataManager(new SQLiteDataManager(URI.create("jdbc:sqlite:" + f.getName())));
		Environment env = new Environment("testlocalenv", new LocalExecutionManager(OutputHandlingOption.DISCARD, OutputHandlingOption.DISCARD));
		
		long time = System.currentTimeMillis();
		AlgorithmRun r = env.fetchRun(pils.getAlgorithmRunRequest());
		Global.getThreadPool().execute(r);
		while (!RunStatus.isFinished(r.getStatus())) {
			System.out.println((System.currentTimeMillis()-time)/1000. + "walls " + r.getTotalCpuTime() + "cpus " + r.getLastOutput());
			try {
				Thread.sleep(10000);
			} catch (InterruptedException ignored) {}
		}
		System.out.println((System.currentTimeMillis()-time)/1000. + "walls " + r.getTotalCpuTime() + "cpus " + r.getLastOutput());
		r.waitForVisitors();
		
		for (Pair<String, LogRecord> lr : Global.getDataManager().getLogRecords(10, 0)) {
			System.out.println(lr.second());
		}
		
		// the only exception should be the warning about duplicate instances
		assertEquals(1, Global.getDataManager().getLogRecords(10, 0).size());
	}
	
	
	@Test
	public void testTransformedTarget() throws Exception {
		ParameterizedAlgorithm saps = null;
		if (System.getProperty("os.name").contains("Mac OS X")) {
			saps = ParameterizedAlgorithm.fromSpec("test/ca/ubc/cs/beta/hal/algorithms/metaalgorithms/design/paramils/oneoffs/saps-continuous.cfg");
		} else {
			System.err.println("Not supported on non-OSX operating systems");
			fail();
		}
		ParameterizedAlgorithm pils = new ExternalParamILSImplementation().getDefaultParameterizedAlgorithm();
		pils.setScenarioValue(AbstractExternalConfiguratorImplementation.TRAINING_SET_SIZE, 1000);
		pils.setScenarioValue(Semantics.MAX_RUNLENGTH, 1);
		pils.setScenarioValue(Semantics.SEED, 0);
		
		PerformanceMetric metric = PerformanceMetric.PAR1;
		InstanceList insts = InstanceList.fromDirectory(new File("test/ca/ubc/cs/beta/hal/algorithms/metaalgorithms/design/paramils/oneoffs"), ".*[.]cnf", "medinst", false, "cnf");
		
		ParameterSettingBuilder psb = new ParameterSettingBuilder();
		psb.put(Semantics.MAX_SUBRUN_CPUTIME, 5.0);
		InstanceMetricMetaProblemInstance mpi = new InstanceMetricMetaProblemInstance(saps, insts, metric, "testbreakinst", psb.build());
		pils.setProblemInstance(mpi);
		
		File f = File.createTempFile("saps", ".testdb", new File("."));
		f.deleteOnExit();
		Global.setDataManager(new SQLiteDataManager(URI.create("jdbc:sqlite:" + f.getName())));
		Environment env = new Environment("testlocalenv", new LocalExecutionManager(OutputHandlingOption.DISCARD, OutputHandlingOption.DISCARD));
		
		AlgorithmRun r = env.fetchRun(pils.getAlgorithmRunRequest());
		Global.getThreadPool().execute(r);
		r.waitForVisitors();
		assertEquals(Double.valueOf(1.01), ((Algorithm)r.getLastOutputValueOnly(Semantics.DESIGN)).getConfiguration().get("alpha"));
	}
	
}
