package ca.ubc.cs.beta.hal.algorithms.metaalgorithms.design.paramils.oneoffs;

import static org.junit.Assert.fail;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.junit.Test;

import ca.ubc.cs.beta.hal.algorithms.ParameterizedAlgorithm;
import ca.ubc.cs.beta.hal.algorithms.metaalgorithms.design.externalconfigurators.AbstractExternalConfiguratorImplementation;
import ca.ubc.cs.beta.hal.algorithms.metaalgorithms.design.paramils.ExternalParamILSImplementation;
import ca.ubc.cs.beta.hal.algorithms.parameters.ParameterSetting;
import ca.ubc.cs.beta.hal.algorithms.parameters.ParameterSettingBuilder;
import ca.ubc.cs.beta.hal.algorithms.parameters.Semantics;
import ca.ubc.cs.beta.hal.environments.AlgorithmRun;
import ca.ubc.cs.beta.hal.problems.InstanceList;
import ca.ubc.cs.beta.hal.problems.InstanceMetricMetaProblemInstance;
import ca.ubc.cs.beta.hal.problems.metrics.PerformanceMetric;

public class SPEARBugTest {
	@Test
	public void testSPEARBug() throws Exception {
		ParameterizedAlgorithm saps = null;
		if (System.getProperty("os.name").contains("Mac OS X")) {
			saps = ParameterizedAlgorithm.fromSpec("test/ca/ubc/cs/beta/hal/algorithms/metaalgorithms/design/paramils/oneoffs/spear-discrete.cfg");
		} else {
			System.err.println("Not supported on non-OSX operating systems");
			fail();
		}
		ParameterizedAlgorithm pils = new ExternalParamILSImplementation().getDefaultParameterizedAlgorithm();
		pils.setScenarioValue(AbstractExternalConfiguratorImplementation.TRAINING_SET_SIZE, 1000);
		pils.setScenarioValue(Semantics.MAX_CPUTIME, 1800);
		pils.setScenarioValue(Semantics.SEED, 0);
		
		PerformanceMetric metric = PerformanceMetric.PAR1;
		InstanceList insts = InstanceList.fromDirectory(new File("test/ca/ubc/cs/beta/hal/algorithms/metaalgorithms/design/paramils/oneoffs"), ".*[.]cnf", "medinst", false, "cnf");
		
		ParameterSettingBuilder psb = new ParameterSettingBuilder();
		psb.put(Semantics.MAX_SUBRUN_CPUTIME, 5.0);
		InstanceMetricMetaProblemInstance mpi = new InstanceMetricMetaProblemInstance(saps, insts, metric, "testbreakinst", psb.build());
		pils.setProblemInstance(mpi);
		
		AlgorithmRun r = ((ExternalParamILSImplementation)pils.getImplementation()).getRun(pils.getAlgorithmRunRequest(), 
				null, null, null);
		r.getAlgorithmRunRequest().setId(null, 1L);
		
		Method m = r.getClass().getSuperclass().getDeclaredMethod("getTargetRunRequest", 
				String.class, int.class, double.class, int.class, ParameterSetting.class);
		m.setAccessible(true);
		try {
			m.invoke(r, "1", 0, 5.0, 1156561, saps.getConfiguration());
		} catch (InvocationTargetException e) {
			// this used to fail with an NPE
			fail();
		}
	}
}
