package ca.ubc.cs.beta.smac;

import static org.junit.Assert.*;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.util.*;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.commons.io.FileUtils;
import org.junit.Test;

import ca.ubc.cs.beta.hal.algorithms.Algorithm;
import ca.ubc.cs.beta.hal.algorithms.ExternalAlgorithmImplementation;
import ca.ubc.cs.beta.hal.algorithms.MetaAlgorithmImplementation;
import ca.ubc.cs.beta.hal.algorithms.MetaAlgorithmImplementation.MetaAlgorithmRun;
import ca.ubc.cs.beta.hal.algorithms.ParameterizedAlgorithm;
import ca.ubc.cs.beta.hal.algorithms.ParameterlessAlgorithm;
import ca.ubc.cs.beta.hal.algorithms.metaalgorithms.design.DesignMetaAlgorithmImplementation;
import ca.ubc.cs.beta.hal.algorithms.metaalgorithms.design.DesignMetaAlgorithmImplementation.RunnableDesign;
import ca.ubc.cs.beta.hal.algorithms.parameters.ParameterSettingBuilder;
import ca.ubc.cs.beta.hal.algorithms.parameters.Semantics;
import ca.ubc.cs.beta.hal.environments.AlgorithmRun;
import ca.ubc.cs.beta.hal.environments.AlgorithmRun.RunStatus;
import ca.ubc.cs.beta.hal.environments.Environment;
import ca.ubc.cs.beta.hal.environments.ExecutionManager.OutputHandlingOption;
import ca.ubc.cs.beta.hal.environments.FullAccessDataManager;
import ca.ubc.cs.beta.hal.environments.datamanagers.DataManagerFactory;
import ca.ubc.cs.beta.hal.environments.datamanagers.SQLiteDataManager;
import ca.ubc.cs.beta.hal.environments.executionmanagers.LocalExecutionManager;
import ca.ubc.cs.beta.hal.problems.*;
import ca.ubc.cs.beta.hal.problems.metrics.PerformanceMetric;
import ca.ubc.cs.beta.hal.utils.Global;
import ca.ubc.cs.beta.smac.ROARConfiguratorImplementation;

public class ROARConfiguratorImplementationTest {
	
	protected MetaAlgorithmImplementation alg = new ROARConfiguratorImplementation();
	private final String cp = Global.isWindows() ? "bin;lib/*;../hal/bin;../HAL/bin" : "bin:lib/*:../hal/bin:../HAL/bin";
	
	static {
		Logger.getLogger("ca.ubc.cs.beta.hal").setLevel(Level.WARNING);
		Logger.getLogger(ROARConfiguratorImplementation.class.getCanonicalName()).setLevel(Level.WARNING);
//		Logger.getLogger("ca.ubc.cs.beta.hal.environments.datamanagers").setLevel(Level.ALL);
	}
	
	@Test
	public void makeSpec() throws IOException {
//		JSONArray a = new JSONArray();
//		a.add(new ROARConfiguratorImplementation().toSpec());
		FileUtils.writeStringToFile(new File("roar.alg"), new ROARConfiguratorImplementation().getDefaultParameterizedAlgorithm().toSpec());
		System.out.println(new ROARConfiguratorImplementation().toSpec());
		System.out.println(new ROARConfiguratorImplementation().getHash());
	}
	
//	@Test
	public void testRepeatable() throws Exception {
//		System.out.println(DataManagerFactory.getDataManager(URI.create("jdbc:sqlite:test/pils.db")));
		
		SQLiteDataManager dm = new SQLiteDataManager(URI.create("jdbc:sqlite:test/roar.db"));
		dm.clear();
		Global.setDataManager(dm);
		
		LocalExecutionManager lem = new LocalExecutionManager(OutputHandlingOption.ECHO, OutputHandlingOption.ECHO);
		final Environment env = new Environment("local unittest", lem);
						
		// get the meta-algorithm
		ParameterizedAlgorithm configurator = alg.getDefaultParameterizedAlgorithm();
		
		String json = "{'command':'java', 'name':'target', 'tags':[['foop']], 'outputFormat':{'stdout':['foroutput $v$ $t$']}, 'outputs':{'v':{'domain':'Real()'}, 't':{'domain':'Real()'}}, 'name':'json1', 'deterministic':false, 'inputFormat':{'callstring':['-classpath " + cp + " ca.ubc.cs.beta.hal.utils.TimeWaster $time$ [$hitime$]']}, 'inputs':{'time':{'domain':'Real[0, 0.5]', 'default':0.25}, 'hitime':{'domain':'Real[0.5, 1]', 'scale':'LOG', 'default':0.75}}}";
		ParameterizedAlgorithm alg1 = ParameterizedAlgorithm.fromSpec(json);
		
//		FileUtils.writeStringToFile(new File("pils.alg"), configurator.toSpec());
		
		PerformanceMetric metric = PerformanceMetric.PAR1;
		
		// create some dummy instances for our dummy algorithm
		LinkedList<ProblemInstance> il = new LinkedList<ProblemInstance>();
		for (int i=0; i<4; i++) {
			File f = File.createTempFile("dummy", ".inst", new File("."));
			f.deleteOnExit();
			FileUtils.writeStringToFile(f, f.getPath());
			il.add(new FileProblemInstance(f));
		}
		InstanceList instances = new InstanceList("dummylist", il);
		instances.addTagsToAllInstances("foop");
		
		ParameterSettingBuilder psb = new ParameterSettingBuilder();
		psb.put(Semantics.MAX_SUBRUN_CPUTIME, 1.0);
		MetaProblemInstance inst = new InstanceMetricMetaProblemInstance(alg1, instances, metric, "dummyConfigProb", psb.build());
		
//		FileUtils.writeStringToFile(new File("sample.designinst"), inst.toSpec());
				
		configurator.setProblemInstance(inst);
		configurator.setScenarioValue(Semantics.MAX_CPUTIME, 1);
		
		AlgorithmRun run = env.fetchRun(configurator.getAlgorithmRunRequest());
		Global.getThreadPool().execute(run);
		run.waitForVisitors();
		
		Iterable<? extends AlgorithmRun> subruns1 = run.getSubruns();
		long src = run.getSubrunCount();
		dm.clear();
		
		run = env.fetchRun(configurator.getAlgorithmRunRequest());
		Global.getThreadPool().execute(run);
		run.waitForVisitors();
		
		Iterable<? extends AlgorithmRun> subruns2 = run.getSubruns();
		
		assertEquals(src, run.getSubrunCount());

		
//		assertEquals(subruns1.size(), subruns2.size());
		Iterator<? extends AlgorithmRun> it1 = subruns1.iterator(), it2 = subruns2.iterator();
		while(it1.hasNext()) {
			assertEquals(it1.next().getAlgorithmRunRequest(),it2.next().getAlgorithmRunRequest());
		}

	}
	
	@Test
	public void testRuns() throws Exception {
		
//		System.out.println(DataManagerFactory.getDataManager(URI.create("jdbc:sqlite:test/pils.db")));
		
		FullAccessDataManager dm = DataManagerFactory.getDataManager(URI.create("jdbc:sqlite:test/pils.db"));
		dm.clear();
		Global.setDataManager(dm);
		
		LocalExecutionManager lem = new LocalExecutionManager(OutputHandlingOption.DISCARD, OutputHandlingOption.ECHO);
		final Environment env = new Environment("local unittest", lem);
						
		// get the meta-algorithm
		ParameterizedAlgorithm configurator = alg.getDefaultParameterizedAlgorithm();
		
		String json = "{'command':'java', 'name':'target', 'tags':[['foop']], 'outputFormat':{'stdout':['foroutput $v$ $t$']}, 'outputs':{'v':{'domain':'Real()'}, 't':{'domain':'Real()'}}, 'name':'json1', 'deterministic':false, 'inputFormat':{'callstring':['-classpath " + cp + " ca.ubc.cs.beta.hal.utils.TimeWaster $time$ [$hitime$]']}, 'inputs':{'time':{'domain':'Real[0, 0.5]', 'default':0.25}, 'hitime':{'domain':'Real[0.5, 1]', 'scale':'LOG', 'default':0.75}}}";
		ParameterizedAlgorithm alg1 = ParameterizedAlgorithm.fromSpec(json);
		
//		FileUtils.writeStringToFile(new File("pils.alg"), configurator.toSpec());
		
		PerformanceMetric metric = PerformanceMetric.PAR1;
		
		// create some dummy instances for our dummy algorithm
		LinkedList<ProblemInstance> il = new LinkedList<ProblemInstance>();
		for (int i=0; i<4; i++) {
			File f = File.createTempFile("dummy", ".inst", new File("."));
			f.deleteOnExit();
			FileUtils.writeStringToFile(f, f.getPath());
			il.add(new FileProblemInstance(f));
		}
		InstanceList instances = new InstanceList("dummylist", il);
		instances.addTagsToAllInstances("foop");
		
		alg1.setConfigurationValue("time", 0.3);
		alg1.setConfigurationValue("hitime", 1.0);
		alg1.setProblemInstance(instances.get(1));
		System.out.println(((ExternalAlgorithmImplementation)alg1.getImplementation()).getCommandString(alg1.getAlgorithmRunRequest()));
		
		ParameterSettingBuilder psb = new ParameterSettingBuilder();
		psb.put(Semantics.MAX_SUBRUN_CPUTIME, 1.0);
		MetaProblemInstance inst = new InstanceMetricMetaProblemInstance(alg1, instances, metric, "dummyConfigProb", psb.build());
		
//		FileUtils.writeStringToFile(new File("sample.designinst"), inst.toSpec());
				
		configurator.setProblemInstance(inst);
		configurator.setScenarioValue(Semantics.MAX_CPUTIME, 20);
		
		AlgorithmRun run = env.fetchRun(configurator.getAlgorithmRunRequest());
		Global.getThreadPool().execute(run);
		assertTrue(run instanceof MetaAlgorithmRun);
		run.waitForVisitors();
		assertTrue(RunStatus.isFinished(run.getStatus()));
				
		assertFalse(RunStatus.finishedWithUnexpectedError(run.getStatus()));
		
		ParameterlessAlgorithm design = (ParameterlessAlgorithm) run.getLastOutputValueOnly(Semantics.DESIGN);
		
		System.out.println(run.getOutput());
		System.out.println(inst.getAlgorithms());
		
		assertEquals(design.getImplementation(), ((Algorithm)inst.getAlgorithms()).getImplementation());
		System.out.println("CFG" + design.getConfiguration());

		System.out.println("total " + run.getTotalCpuTime());
		System.out.println("measured " + run.getMeasuredCpuTime());
		System.out.println("overhead " + run.getOverheadCpuTime());
		System.out.println("subs " + run.getSubrunTotalCpuTime());
		System.out.println("subM " + run.getSubrunMeasuredCpuTime());
		System.out.println("subO " + run.getSubrunOverheadCpuTime());
		
		RunnableDesign x = ((DesignMetaAlgorithmImplementation)configurator.getImplementation()).exportDesign(run, File.createTempFile("roardesign", ".zip"));
		System.out.println(x.getCommandIfExtracted());
		synchronized(x.getExportJob()) {
			Global.getThreadPool().execute(x.getExportJob());
			while (x.getExportJob().getFractionCompleted() < 1) {
				x.getExportJob().wait();
			}
		}
		System.out.println(x.getExportJob().getFile());
		x.getExportJob().getFile().delete();
		
//		// validate
//		InstanceMetricMetaProblemInstance validationInst = new InstanceMetricMetaProblemInstance(
//				new ParameterlessAlgorithmList(design), instances, metric, 
//				"dummyValidationInstance");
//		validationInst.setOption(Semantics.MAX_SUBRUN_CPUTIME, 1.0);
////		validationInst.setOption(Semantics.RUNS_PER_INSTANCE, 10);
//		ParameterizedAlgorithm alg2 = new SingleAlgorithmAnalysisMetaAlgorithmImplementation().getDefaultParameterizedAlgorithm();
//		alg2.setProblemInstance(validationInst);
//		alg2.setScenarioValue(Semantics.MAX_RUNLENGTH, 20);
//		AlgorithmRun validationRun = env.fetchRun(alg2.getAlgorithmRunRequest());
//		Global.getThreadPool().execute(validationRun);
//		validationRun.waitFor();
		
// Commented out by FH to run successfully on Windows.
//		Plotter p = new GnuplotPlotter(GnuplotPlotter.DFLT_PROPS);
//		System.out.println(validationRun.getLastOutputValueOnly());
//		p.display((Plot)validationRun.getLastOutputValueOnly(SingleAlgorithmAnalysisMetaAlgorithmImplementation.SCD));
		
//		// wait for everything to commit.
//		do {
//			Thread.sleep(200);
//		} while (dm.getActiveConnections() > 0);
//		
//		// should come from the DB
//		for (int i=0; i<10; i++) {
//			run = env.fetchRun(configurator.getAlgorithmRunRequest());
//			assertTrue(run instanceof DatabaseAlgorithmRun);
//		}
	}
}
